import './style.css'
import { generateIcon } from './functions'
import React, { forwardRef, useImperativeHandle, useRef } from 'react'

const TextPlaceholder = ({ data }) => <div style={{ width: 'max-content', marginRight: '20px', marginLeft: '20px' }}>{data}</div>

const Field = forwardRef((props, ref) => {

    const { placeholder, text, disabled = false, isTwo, type, callbacks: { f: func = () => {} } = {} } = props?.data


    const inputRef = useRef(null);
    const _icon = generateIcon(type);

    const shadow = {
        true:  `inset 0px 7px 10px -7px rgba(0,0,0,0.8), inset 7px 0px 10px -7px rgba(135,155,193,0.8), inset -7px 0px 10px -7px rgba(135,155,193,0.8), inset 0px -7px 10px -7px rgba(0,0,0,0.8)`,
        false: `inset 0px 7px 10px -7px rgba(0,0,0,0.8), inset 7px 0px 10px -7px rgba(255,255,255,0.8), inset -7px 0px 10px -7px rgba(255,255,255,0.8), inset 0px -7px 10px -7px rgba(0,0,0,0.8)`
    }

    const _style = {
        text:   { backgroundColor: (disabled) ? '#879bc1' : 'white', boxShadow: (disabled) ? shadow['true'] : shadow['false'], color: (disabled) ? 'grey' : 'black' },
        input:  { backgroundColor: (disabled) ? '#879bc1' : 'white', boxShadow: (disabled) ? shadow['true'] : shadow['false'], color: (disabled) ? 'grey' : 'black' },
        icon:   { backgroundColor: (disabled) ? '#879bc1' : 'white', boxShadow: (disabled) ? shadow['true'] : shadow['false'], color: (disabled) ? 'grey' : 'blue', marginRight: isTwo && '20px' },
    }

    useImperativeHandle(ref, () => inputRef.current,[])
    
    return (
        <div className="field-wrapper flex h55p maxw775 w100 align-ct" onClick={func}>
            <div className="field-text flex align-ct justify-fs" style={_style['text']}><TextPlaceholder data={placeholder}/></div>
            <div className="field-input flex w100 center" style={_style['input']} ref={inputRef}>{text || ''}</div>
            <div className="field-icon flex center" style={_style['icon']}>{_icon}</div>
        </div>
    )
})

export default Field;