import React from 'react';
import { OK, NOK, DEL } from './Icons';

export default function Key(key, funcs) {
	const { setKeyboardText, setKeyboardMode, text, id } = funcs;

	const shiftkey  = (mode) => setKeyboardMode(mode);
	const spacekey  = () => setKeyboardText(prev => `${prev} `);
	const simplekey = (keyName) => setKeyboardText(prev => prev + keyName);
	const delkey    = () => setKeyboardText(prev => prev.replace(/.$/, ''));
	const nokkey    = () => window.api.renderer({ mode: 'screen', type: 'keyboard', visible: false });
	const okkey     = () => { 
        window.api.renderer({ mode: 'screen', type: 'handleKeyboard', data: { id, value: text }});
        window.api.renderer({ mode: 'screen', type: 'keyboard', visible: false });
	}

	const keyData = [
		[
			{ keyName: 'q', f: () => simplekey('q'), keyMarginLeft: 'auto' },
			{ keyName: 'w', f: () => simplekey('w') },
			{ keyName: 'e', f: () => simplekey('e') },
			{ keyName: 'r', f: () => simplekey('r') },
			{ keyName: 't', f: () => simplekey('t') },
			{ keyName: 'y', f: () => simplekey('y') },
			{ keyName: 'u', f: () => simplekey('u') },
			{ keyName: 'i', f: () => simplekey('i') },
			{ keyName: 'o', f: () => simplekey('o') },
			{ keyName: 'p', f: () => simplekey('p'), keyMarginRight: 'auto' },

			{ keyName: 'a', f: () => simplekey('a'), keyMarginLeft: 'auto' },
			{ keyName: 's', f: () => simplekey('s') },
			{ keyName: 'd', f: () => simplekey('d') },
			{ keyName: 'f', f: () => simplekey('f') },
			{ keyName: 'g', f: () => simplekey('g') },
			{ keyName: 'h', f: () => simplekey('h') },
			{ keyName: 'j', f: () => simplekey('j') },
			{ keyName: 'k', f: () => simplekey('k') },
			{ keyName: 'l', f: () => simplekey('l') },
			{ keyName: 'ç', f: () => simplekey('ç'), keyMarginRight: 'auto' },

			{
				keyName: 'shift',
				f: () => shiftkey(1),
				keyWidth: '165px',
				keyMarginLeft: 'auto'
			},
			{ keyName: 'z', f: () => simplekey('z') },
			{ keyName: 'x', f: () => simplekey('x') },
			{ keyName: 'c', f: () => simplekey('c') },
			{ keyName: 'v', f: () => simplekey('v') },
			{ keyName: 'b', f: () => simplekey('b') },
			{ keyName: 'n', f: () => simplekey('n') },
			{ keyName: 'm', f: () => simplekey('m') },
			{
				keyName: 'del',
				f: delkey,
				keyWidth: '165px',
				icon: <DEL />,
				keyMarginRight: 'auto',
				length: text.length,
			},

			{
				keyName: 'nok',
				f: nokkey,
				keyWidth: '165px',
				icon: <NOK />,
				color: '#e8292a',
				keyMarginLeft: 'auto'
			},
			{ keyName: '123', f: () => shiftkey(2), keyWidth: '120px' },
			{ keyName: ' ', f: spacekey, keyWidth: '35%' },
			{ keyName: '.', f: () => simplekey('.') },
			{
				keyName: 'ok',
				f: okkey,
				keyWidth: '165px',
				icon: <OK />,
				color: '#3cc900',
				keyMarginRight: 'auto'
			}
		],
		[
			{ keyName: 'Q', f: () => simplekey('Q'), keyMarginLeft: 'auto' },
			{ keyName: 'W', f: () => simplekey('W') },
			{ keyName: 'E', f: () => simplekey('E') },
			{ keyName: 'R', f: () => simplekey('R') },
			{ keyName: 'T', f: () => simplekey('T') },
			{ keyName: 'Y', f: () => simplekey('Y') },
			{ keyName: 'U', f: () => simplekey('U') },
			{ keyName: 'I', f: () => simplekey('I') },
			{ keyName: 'O', f: () => simplekey('O') },
			{ keyName: 'P', f: () => simplekey('P'), keyMarginRight: 'auto' },

			{ keyName: 'A', f: () => simplekey('A'), keyMarginLeft: 'auto' },
			{ keyName: 'S', f: () => simplekey('S') },
			{ keyName: 'D', f: () => simplekey('D') },
			{ keyName: 'F', f: () => simplekey('F') },
			{ keyName: 'G', f: () => simplekey('G') },
			{ keyName: 'H', f: () => simplekey('H') },
			{ keyName: 'J', f: () => simplekey('J') },
			{ keyName: 'K', f: () => simplekey('K') },
			{ keyName: 'L', f: () => simplekey('L') },
			{ keyName: 'Ç', f: () => simplekey('Ç'), keyMarginRight: 'auto' },

			{
				keyName: 'SHIFT',
				f: () => shiftkey(0),
				keyWidth: '165px',
				keyMarginLeft: 'auto'
			},
			{ keyName: 'Z', f: () => simplekey('Z') },
			{ keyName: 'X', f: () => simplekey('X') },
			{ keyName: 'C', f: () => simplekey('C') },
			{ keyName: 'V', f: () => simplekey('V') },
			{ keyName: 'B', f: () => simplekey('B') },
			{ keyName: 'N', f: () => simplekey('N') },
			{ keyName: 'M', f: () => simplekey('M') },
			{
				keyName: 'del',
				f: delkey,
				keyWidth: '165px',
				icon: <DEL />,
				keyMarginRight: 'auto',
				length: text.length,
			},

			{
				keyName: 'nok',
				f: nokkey,
				keyWidth: '165px',
				icon: <NOK />,
				color: '#e8292a',
				keyMarginLeft: 'auto'
			},
			{ keyName: '123', f: () => shiftkey(2), keyWidth: '120px' },
			{ keyName: ' ', f: spacekey, keyWidth: '35%' },
			{ keyName: '.', f: () => simplekey('.') },
			{
				keyName: 'ok',
				f: okkey,
				keyWidth: '165px',
				icon: <OK />,
				color: '#3cc900',
				keyMarginRight: 'auto'
			}
		],
		[
			{ keyName: '1', f: () => simplekey('1'), keyMarginLeft: 'auto' },
			{ keyName: '2', f: () => simplekey('2') },
			{ keyName: '3', f: () => simplekey('3') },
			{ keyName: '4', f: () => simplekey('4') },
			{ keyName: '5', f: () => simplekey('5') },
			{ keyName: '6', f: () => simplekey('6') },
			{ keyName: '7', f: () => simplekey('7') },
			{ keyName: '8', f: () => simplekey('8') },
			{ keyName: '9', f: () => simplekey('9') },
			{ keyName: '0', f: () => simplekey('0'), keyMarginRight: 'auto' },

			{ keyName: '-', f: () => simplekey('-'), keyMarginLeft: 'auto' },
			{ keyName: '/', f: () => simplekey('/') },
			{ keyName: ':', f: () => simplekey(':') },
			{ keyName: ';', f: () => simplekey(';') },
			{ keyName: '(', f: () => simplekey('(') },
			{ keyName: ')', f: () => simplekey(')') },
			{ keyName: '$', f: () => simplekey('$') },
			{ keyName: '&', f: () => simplekey('&') },
			{ keyName: '@', f: () => simplekey('@') },
			{ keyName: '"', f: () => simplekey('"'), keyMarginRight: 'auto' },

			{
				keyName: '#+=',
				f: () => shiftkey(3),
				keyWidth: '165px',
				keyMarginLeft: 'auto'
			},
			{ keyName: '.', f: () => simplekey('.') },
			{ keyName: ',', f: () => simplekey(',') },
			{ keyName: '?', f: () => simplekey('?') },
			{ keyName: '!', f: () => simplekey('!') },
			{ keyName: '|', f: () => simplekey('|') },
			// { keyName: "\'", f: () => simplekey("\'") },
			{
				keyName: 'del',
				f: delkey,
				keyWidth: '165px',
				icon: <DEL />,
				keyMarginRight: 'auto',
				length: text.length,
			},

			{
				keyName: 'nok',
				f: nokkey,
				keyWidth: '165px',
				icon: <NOK />,
				color: '#e8292a',
				keyMarginLeft: 'auto'
			},
			{ keyName: 'ABC', f: () => shiftkey(0), keyWidth: '120px' },
			{ keyName: ' ',   f: spacekey, keyWidth: '35%' },
			{ keyName: '.',   f: () => simplekey('.') },
			{
				keyName: 'ok',
				f: okkey,
				keyWidth: '165px',
				icon: <OK />,
				color: '#3cc900',
				keyMarginRight: 'auto'
			}
		],
		[
			{ keyName: '[',  f: () => simplekey('['), keyMarginLeft: 'auto' },
			{ keyName: ']',  f: () => simplekey(']') },
			{ keyName: '{',  f: () => simplekey('{') },
			{ keyName: '}',  f: () => simplekey('}') },
			{ keyName: '#',  f: () => simplekey('#') },
			{ keyName: '%',  f: () => simplekey('%') },
			{ keyName: '^',  f: () => simplekey('^') },
			{ keyName: '*',  f: () => simplekey('*') },
			{ keyName: '+',  f: () => simplekey('+') },
			{ keyName: '=',  f: () => simplekey('='), keyMarginRight: 'auto' },

			{ keyName: '_',  f: () => simplekey('_'), keyMarginLeft: 'auto' },
			{ keyName: '\\', f: () => simplekey('\\') },
			{ keyName: '|',  f: () => simplekey('|') },
			{ keyName: '~',  f: () => simplekey('~') },
			{ keyName: '<',  f: () => simplekey('<') },
			{ keyName: '>',  f: () => simplekey('>'), keyMarginRight: 'auto' },

			{
				keyName: '123',
				f: () => shiftkey(2),
				keyWidth: '165px',
				keyMarginLeft: 'auto'
			},
			{ keyName: '.', f: () => simplekey('.') },
			{ keyName: ',', f: () => simplekey(',') },
			{ keyName: '?', f: () => simplekey('?') },
			{ keyName: '!', f: () => simplekey('!') },
			{ keyName: '|',  f: () => simplekey('|') },
			// { keyName: "\'", f: () => simplekey("\'") },
			{
				keyName: 'del',
				f: delkey,
				keyWidth: '165px',
				icon: <DEL />,
				keyMarginRight: 'auto',
				length: text.length,
			},

			{
				keyName: 'nok',
				f: nokkey,
				keyWidth: '165px',
				icon: <NOK />,
				color: '#e8292a',
				keyMarginLeft: 'auto'
			},
			{ keyName: 'ABC', f: () => shiftkey(0), keyWidth: '120px' },
			{ keyName: ' ',   f: spacekey, keyWidth: '35%' },
			{ keyName: '.',   f: () => simplekey('.') },
			{
				keyName: 'ok',
				f: okkey,
				keyWidth: '165px',
				icon: <OK />,
				color: '#3cc900',
				keyMarginRight: 'auto'
			}
		]
	]

	return keyData[key]
}
