import React from "react"

const translate = window.language.translate
import { BACK_REPORT as BACK } from '../../../../../icons';

const Value = (param) => (param !== null && param !== undefined) ? param : 'XXXXXXXX';

export const Wrapper = ({ children }) => <div className="report-wrapper">{children}</div>
export const Content = ({ children }) => <div className="report-content">{children}</div>
export const Left    = ({ children }) => <div className="report-left-side">{children}</div>
export const Right   = ({ children }) => <div className="report-right-side">{children}</div>
export const Image   = ({ children, data: { image, onClick } }) => <div className="report-image" style={{ backgroundImage: (image !== 'erro') ? `url(${image})` : 'none' }} onClick={onClick} >{children}</div>
export const Buttons = ({ children }) => <div className="report-container-buttons">{children}</div>
export const Table   = ({ children }) => <div className="report-table">{children}</div>

export const QRCodeMini = () => <div className="report-qrcode-mini"></div>
export const QRCode  = ({ data: { qrcode }}) => <div className="report-qrcode" style={{ backgroundImage: (qrcode !== 'erro') ? `url(${qrcode})` : 'none' }}></div>

export function LineTwo(props) {
    const { textOne, valueOne, textTwo, valueTwo } = props.data

    return (
        <div className="line-two-report-wrapper">
            <div className="line-two-report-label">
                <div className="line-two-report-text">{textOne}</div>
                <div className="line-two-report-value">{Value(valueOne)}</div>
            </div>
            <div className="line-two-report-label">
                <div className="line-two-report-text">{textTwo}</div>
                <div className="line-two-report-value">{Value(valueTwo)}</div>
            </div>
        </div>
    )
}

export function LineOne(props) {

    const { text, value } = props['data'];

    return (
        <div className="line-one-report-wrapper">
            <div className="line-one-report-label">
                <div className="line-one-report-text">{text}</div>
                <div className="line-one-report-value">{Value(value)}</div>
            </div>
        </div>
    )
}

export function Button(props) {
    const { style, func, icon } = props.data
    return (
        <div className="button-report-wrapper"
            onClick={func}
            style={{ 
                fontSize: `${style.font}`,
                width: `${style.length}`,
                opacity: `${(style?.disabled) ? (style?.disabled === translate(103) || style?.disabled === translate(52)) ? 1 : 0.5 : 1}`}}
        >{icon}</div>
    )
}

export const QRCodeWrapper = ({ data: { qrcode, fnc }}) => <div className="report-qrcode-wrapper">
    <div className="report-qrcode-content">
        <QRCode data={{ qrcode }} />
        <div className="report-qrcode-content-button">
            <Button data={{ func: () => fnc(false),  icon: <BACK />, style: { length: '80px' }}} />
        </div>
    </div>
</div>
