import './style.css';
import React from 'react';
import Capsula from '../../common/Capsula';
import { Line, Canceled, Positive, Negative, QRCode } from './components';

export default function Result(props) {

	const translate = window.language.translate

	const { number, type, time, image, detected, canceled, percent, state, color, qrcode } = props.data

	const _image = (e) => showZoom(e, { slot: number, type, image, detected })
	const _cap = { number, canOpen: false, percent, state, color, canceled, type: false }
	
	const close = () => window.api.send('RENDER/RESULT', { mode: 'popup', type: 'result', visible: false })

	const LINE = {
		time: 	{ text: translate(75), value: time },
		cavity: { text: translate(74), value: number },
		type: 	{ text: translate(48), value: translate(type) },
	}

	function showZoom(e, {slot, type, image, detected}) {
		e.stopPropagation();
		if (detected) window.api.renderer({ mode: 'popup', type: 'zoom', visible: true, data: { slot, type, image }})
	}

	return (
		<div className="result-wrapper">
			<div className="result-content" onClick={close}>
				<div className="result-row">
					<div className='flex'>
						<div>
							<Line data={LINE['cavity']} />
							<Line data={LINE['type']} />
							<Line data={LINE['time']} />
						</div>
						<QRCode data={{ qrcode }} />
					</div>
					{(canceled) ? <Canceled /> : (detected) ? <Positive /> : <Negative />}
				</div>
				<div className="result-image" 
					style={{ backgroundImage: (image !== 'erro') ? `url(${image})` : 'none' }} 
					onClick={_image}>
					{canceled && <div className="result-canceled">X</div>}
				</div>
				<Capsula data={_cap} />
			</div>
		</div>
	)
}
