import React, { useState, useEffect } from 'react';
import { Wrapper, Content, Header, Switch } from './components';
import { Time, Text, Button, Buttons, Container } from './components';

export default function Backup(props) {

    const { isON, time, last_time } = props.data;
    const { hour, minute } = time;

    const HOURSINDAY = 24;
    const MINUTESINHOUR = 60;
    const translate = window.language.translate

    const [selectedHour,    setSelectedHour]   = useState(0);
    const [selectedMinute,  setSelectedMinute] = useState(0);
    const [hours,           setHours]          = useState([]);
    const [minutes,         setMinutes]        = useState([]);
    const [isSelected,      setSelected]       = useState(isON);

    const CONFIG = {
        hours:   { array: hours,   type: 'hour',   func: ({ value }) => setSelectedHour(value),   begin: hour },
        minutes: { array: minutes, type: 'minute', func: ({ value }) => setSelectedMinute(value), begin: minute },
    }

    const close   = () => window.api.renderer().close('settings', 'backup');
    const confirm = () => {

        const hour =   selectedHour.toString().padStart(2,0);
        const minute = selectedMinute.toString().padStart(2,0);
    
        window.api.send('BACKUPAUTOMATIC', { isON: isSelected, time: { hour, minute }});
        close();
    }

    useEffect(() => {
        setSelectedHour(hour);
        setSelectedMinute(minute);
        for (let i = 0; i < HOURSINDAY; i++)    setHours(prev   => [...prev, i]);
        for (let i = 0; i < MINUTESINHOUR; i++) setMinutes(prev => [...prev, i]);
        // eslint-disable-next-line
    }, []);


    return (
        <Wrapper data={{ z_index: 50 }}>
            <Content>
                <Container>
                    <Header />
                    <Switch data={{ isON: isSelected, func: setSelected, height: 30, width: 40 }}/>
                    <Time data={CONFIG} />
                    <Text data={{ text: translate(173).split('0')[0], date: last_time }} />
                </Container>
                <Buttons>
                    <Button data={{ func: close,   text: translate(17) }} />
                    <Button data={{ func: confirm, text: translate(22) }} />
                </Buttons>
            </Content>
        </Wrapper>
    );
}