import React from 'react';
import styled from 'styled-components';
import * as style from '../../../../../style';

export const Content = styled.div`
    gap: 16px;
    width: 860px;
    height: 580px; 
    padding: 16px;
    display: flex; 
    border-radius: 16px; 
    border: 2px solid white;
    background-image: ${style.BACKGROUND_GRADIENT};
`;

export const Side = styled.div`
    height: 100%;
    padding: 16px;
    display: flex;
    border-radius: 16px;
    flex-direction: column;
    background-color: white;
    justify-content: space-between;
    width: ${props => props?.data?.width || '100%' };
    min-width: ${props => props?.data?.width || '100%' };
`;

export const ButtonsContainer = styled.div`
    gap: 16px;
    width: 100%;
    display: flex;
    align-items: center;
    height: ${props => props?.data?.height || '100%' };
    flex-direction: ${props => props?.data?.direction || 'row' };
`;

export const Button = styled.div`
    color: white;
    display: flex;
    font-size: 12pt;
    font-weight: bold;
    border-radius: 16px;
    flex-direction: row;
    align-items: center;
    border: 2px solid black;
    background-image: ${style.BACKGROUND_GRADIENT};
    width: ${props => props?.data?.width || '100%' };
    height: ${props => props?.data?.height || '50px' };
    justify-content: ${props => props?.data?.justify || 'center' };
`;

export const Info = styled.div`
    width: 100%;
    color: white;
    height: 300px;
    display: flex; 
    border-radius: 16px;
    flex-direction: column;
    align-items: flex-start;
    border: 2px solid black;
    justify-content: space-around;
    background-image: ${style.BACKGROUND_GRADIENT};
`;

export const Line = styled.div`
    width: 100%;
    color: white;
    display: flex; 
    font-size: 18pt;
    padding-left: 16px;
`;

const SwitchWrapper = styled.div`
    display: flex;
    width: 100px;
    height: 100%;
    align-items: center;
    justify-content: center;
`;

const Ball = styled.div`
    width: 90%;
    height: 90%;
    display: flex;
    border-radius: 50%;
    background-image: ${props => (props?.data?.state) ? style.BACKGROUND_GRADIENT : 'none' };
    background-color: ${props => (!props?.data?.state) ? '#dcdcdc' : 'none' };
`;

const Cylinder = styled.div`
    width: 100px;
    height: 50px;
    display: flex;
    align-items: center;
    border-radius: 50px;
    background-color: #b8b8b8;
    padding-left:  ${props => (!props?.data?.state)  ? '2px' : 'none' };
    padding-right: ${props => (props?.data?.state) ? '2px' : 'none' };
`;

const Text = styled.div`
    width: 100%;
    color: black;
    height: 100%;
    display: flex;
    font-size: 16pt;
    align-items: center;
    justify-content: center;
`;

export function Switch({ data: { state, func }}) {
    return <SwitchWrapper onClick={func}>{state ? <ON/> : <OFF/>}</SwitchWrapper>
}

function ON() {
    return (
        <Cylinder data={{ state: true }}>
            <Text>ON</Text>
            <Ball data={{ state: true }} />
        </Cylinder>
    )
}

function OFF() {
    return (
        <Cylinder data={{ state: false }}>
            <Ball data={{ state: false }} />
            <Text>OFF</Text>
        </Cylinder>
    )
}