import './style.css';
import React, { useState } from 'react';

import { Wrapper } from '../../common';

import { Content, Side, Switch } from './components';
import { Info, Line, ButtonsContainer, Button } from './components';
import { BACK, Factory, Language, PRINTER_REPORT as PRINTER } from '../../../../icons';

export default function Config({ data }) {
	
	const translate = window.language.translate
	const sender = window.api.send;

	const { info: { model, version_software, version_firmware, fabrication_date, serial_number, }} = data;
	const { switch_value, setSwitchValue } = data;

	const update 	= () => window.api.send('UPDATE');
	const guide 	= () => window.api.renderer().open('settings', 'guide');
	const language 	= () => window.api.send('LANGUAGE/render');
	const cancel 	= () => window.api.send('RENDER/config', { mode: 'settings', type: 'config', visible: false, });
	const qrcode 	= () => setSwitchValue(prev => {
		sender('QRCODE/change', !prev);
		return !prev;
	});

	const _lines = [
		{ text: 'MOD.', 	value: model },
		{ text: 'NS / SN', 	value: serial_number },
		{ text: 'SOFTWARE', value: version_software },
		{ text: 'FIRMWARE', value: version_firmware },
		{ text: <Factory data={{ h: 25, w: 40 }} />, value: fabrication_date}
	]; 

	const _buttons = [
		{ text: translate(90), 	func: () => sender('DATABASE/Export/CSV'), },
		{ text: translate(91), 	func: () => sender('DATABASE/Export/PDF'), },
		{ text: translate(92), 	func: () => sender('DATABASE/Export') },
		{ text: translate(93), 	func: () => sender('DATABASE/Import', 'import'), },
		{ text: translate(171), func: () => sender('openWindowBackup'), },
		{ text: translate(94), 	func: () => sender('DATABASE/Clear', 'clear'), },
		{ text: translate(96), 	func: () => sender('PRINTER/Test') },
		{ text: translate(154), func: () => sender('CALIBRATE/on', 'calibrate'), },
	];

	return (
		<Wrapper data={{ index: 6 }}>
			<Content>
				<Side data={{ width: '480px' }}>
					<Info>
						{_lines.map(({ text, value }) => (
							<Line key={text}>
								{text} - {value}
							</Line>
						))}
					</Info>
					<ButtonsContainer data={{ height: '85px' }}>
						<Button data={{ height: '85px', width: '252px', justify: 'space-evenly' }} onClick={qrcode}>
							{translate(177)}
							<PRINTER />
							<Switch data={{ state: switch_value, func: () => {}}}/>
						</Button>
						<Button data={{ height: '85px', width: '180px' }} onClick={guide}>
							{translate(176)}
						</Button>
					</ButtonsContainer>
					<ButtonsContainer data={{ height: '85px' }}>
						<Button data={{ height: '85px', width: '98px' }} onClick={cancel}>
							<BACK />
						</Button>
						<Button data={{ height: '85px', width: '138px' }} onClick={update}>
							{translate(164)}
						</Button>
						<Button data={{ height: '85px', width: '180px', justify: 'space-evenly' }} onClick={language}>
							{translate(95)}
							<Language />
						</Button>
					</ButtonsContainer>
				</Side>
				<Side data={{ width: '330px' }}>
					<ButtonsContainer data={{ direction: 'column', justify: 'space-evenly' }}>
						{_buttons.map(({ func, text }) => (
							<Button key={text} onClick={func}>
								{text}
							</Button>
						))}
					</ButtonsContainer>
				</Side>
			</Content>
		</Wrapper>
	);
}
