import "./style.css";
import React, { useState, useEffect } from "react";
import { Pagination, ButtonPage, Table } from "./components";
import { ButtonAction, Content, LowerSide, TableLine } from "./components";
import { TableHeader, TableList, UpperSide, Wrapper } from "./components";

export default function Edit(props) {

	const translate = window.language.translate

	const { array, header, type } = props['data'];

	const [pages, setMaxPages] = useState(0);
	const [currentPage, setCurrentPage] = useState(1);

	const close = () => window.api.renderer().close("settings", "edit");
	const add   = (mode, info = "") => window.api.renderer().open("settings", "input", { type: type, mode, info });
	const edit  = (mode, info)      => window.api.send("EDIT/verify", { mode: "settings", type: "input", visible: true, data: { type: type, mode, info } });

	const nextPage = () => currentPage !== pages && setCurrentPage((prev) => prev + 1);
	const prevPage = () => currentPage !== 1 	 && setCurrentPage((prev) => prev - 1);


	function genArray(item, index) {
		if (index >= (currentPage - 1) * 5 && index < currentPage * 5) {
			return <TableLine key={item['id']} data={{ type, text: translate(16), info: item, f: () => edit("editing", item)}} />
		}
	}


	const CONFIG = {
		action: {
			back: { text: translate(17), f: close },
			new:  { text: translate(18), f: () => add("adding") },
		},
		page: {
            next:  { text: "\\/", f: nextPage },
			prev:  { text: "/\\", f: prevPage },
			atual: { text: (pages == 0) ? '' : `${currentPage} / ${pages}` },
		}
	};

	useEffect(() => {
		setMaxPages(Math.ceil(array.length / 5));
        if (pages > 0) setCurrentPage(pages);
	}, [array, pages]);

	return (
		<Wrapper>
			<Content>
				<UpperSide>
					<Table>
						<TableHeader data={header} />
						<TableList>
                            {(array.length > 0) && array.map((i, index) => genArray(i, index))}
                        </TableList>
					</Table>
					<Pagination>
						<ButtonPage data={CONFIG["page"]["prev"]} />
						<ButtonPage data={CONFIG["page"]["atual"]} />
						<ButtonPage data={CONFIG["page"]["next"]} />
					</Pagination>
				</UpperSide>
				<LowerSide>
					<ButtonAction data={CONFIG["action"]["back"]} />
					<ButtonAction data={CONFIG["action"]["new"]} />
				</LowerSide>
			</Content>
		</Wrapper>
	);
}
