import React, { useEffect } from "react";
import { Wrapper } from "../../common";
import { Button, Content, Title, Image } from "./components";
import { Center, Bottom, Text, Container } from "./components";
import { BACK, ArrowLeft, ArrowRight } from '../../../../icons';

export default function Guide({ data }) {

	const translate = window.language.translate

	const { isFirstSlide, isLastSlide, currentSlide, slides } = data;
	const { setCurrentSlide, setIsFirstSlide, setIsLastSlide } = data;

	const close = () => window.api.renderer().close('settings', 'guide');
	const add = () => (!isLastSlide) && setCurrentSlide(prev => prev + 1);
	const subtract = () => (!isFirstSlide) && setCurrentSlide(prev => prev - 1);

	useEffect(() => {

		if (currentSlide < 0 || currentSlide >= 12) setCurrentSlide(11);

		switch(currentSlide) {
			case 0:
				setIsFirstSlide(true);
			break;
			case 1:
				setIsFirstSlide(false);
			break;
			case 11:
				setIsLastSlide(true);
			break;
			default:
				setIsLastSlide(false);
				setIsFirstSlide(false);
			break;
		}

	}, [currentSlide]);

    return (
        <Wrapper data={{ index: 31 }}>
			<Content>
				<Container>
					<Title>{translate(slides[currentSlide]?.title)}</Title>
					<Center>
						<Button onClick={subtract} data={{ width: '75px', height: '85px', opacity: (isFirstSlide) ? '0' : '1' }}>
							<ArrowLeft />
						</Button>
						<Image data={{ image: slides[currentSlide]?.image }}/>
						<Button onClick={add} data={{ width: '75px', height: '85px', opacity: (isLastSlide) ? '0' : '1' }}>
							<ArrowRight />
						</Button>
					</Center>
					<Bottom>
						<Button onClick={close} data={{ width: '90px', height: '85px' }}>
							<BACK />
						</Button>
						<Text>{translate(slides[currentSlide]?.text)}</Text>
					</Bottom>
				</Container>
			</Content>
        </Wrapper>
    );
}