import './style.css';
import React from 'react';
import Notification from '../Notification';
import { Empty, UpperContainer, LowerContainer } from './components';
import { Wrapper, Button, ButtonWifi, ExitButton } from './components';

import { VolumeLow, VolumeHigh, Autoclave } from '../../../../icons';
import { Wifi, Clock, ThermometerMenu, User2, ArrowUp, Settings } from '../../../../icons';

export default function MenuDown(props) {

	const { sound_enabled, enableSound, network_selected, notification } = props['data'];
	const { render, _data } = notification;

	const closeMenu = () => window.api.renderer().close('popup', 'menu');

	const changeTemperature = () => window.api.send('TEMPERATURE/change');

	const openDate = () => window.api.send('RENDER/DATE');
	const openConfig = () => window.api.send('RENDER/config', { mode: 'settings', type: 'config', visible: true });

	const openWifi = () => window.api.send('RENDER/WIFI', {mode: 'settings', type: 'wifi', visible: true });
	
	const openEdit = (type) => window.api.send('RENDER/EDIT', {mode: 'settings', type: 'edit', visible: true,  data: { header: type }});

	const CONFIG = {
		clock: 	  { icon: <Clock />, 	  		func: openDate },
		settings: { icon: <Settings />,    		func: openConfig },
		termo: 	  { icon: <ThermometerMenu />, 	func: changeTemperature },
		user: 	  { icon: <User2 />, 	  		func: () => openEdit('users') },
		auto: 	  { icon: <Autoclave />,   		func: () => openEdit('equips') },
		wifi: 	  { icon: <Wifi />, 		  	func: openWifi, label: network_selected  },
		volume: { 
			icon: (sound_enabled) ? <VolumeHigh /> : <VolumeLow />,
			func: () => { enableSound(value => {
				window.api.send('SOUND/change', !value);
				return !value;
			}) }
		},
	}

	return (
		<Wrapper>
			<UpperContainer>
				<ButtonWifi data={CONFIG['wifi']}/>
				<Button data={CONFIG['clock']}/>
				<Button data={CONFIG['termo']}/>
				<Button data={CONFIG['user']}/>
				<Button data={CONFIG['auto']}/>
				<Button data={CONFIG['volume']}/>
				<Button data={CONFIG['settings']}/>
			</UpperContainer>
			<LowerContainer>
				{render ? <Notification data={_data} /> : <Empty />}
				<ExitButton data={{ icon: <ArrowUp />, onClick: closeMenu }}/>
			</LowerContainer>
		</Wrapper>
	)
}