import './style.css';
import React, { useState, useEffect } from 'react';
import { Busy, Center, Date, Left, Logo } from './components';
import { WifiOn, WifiOff, WifiError, Warning } from '../../../../icons';
import { Right, Temp, Time, Wifi, Wrapper } from './components';

export default function TopBar() {

    const [temp, setTemp] = useState('0°C');
    const [busy1, setBusy1] = useState(false);
    const [busy2, setBusy2] = useState(false);
    const [wifi, setWifi] = useState('off');
    const [time, setTime] = useState('00:00');
    const [date, setDate] = useState('01/01/2023');
    const [hasNotification, setHasNotification] = useState(false);

    const style1 = { backgroundColor: (busy1) ? 'red' : 'black', borderRadius: '50%' }
    const style2 = { backgroundColor: (busy2) ? 'blue' : 'black', borderRadius: '50%' }

    const renderWifi = (wifi == true) ? <WifiOn/> : (wifi == false) ? <WifiOff /> : <WifiError />

    useEffect(() => {
        
        window.api.update('wifi',        (data) => setWifi(data));
        window.api.update('isBusy',      (data) => setBusy1(data));
        window.api.update('isBusyBlue',      (data) => setBusy2(data));
        window.api.update('hasNotification', (data) => setHasNotification(data));
        window.api.update('date',        ({ date, time }) => { setDate(date); setTime(time) });
        window.api.update('temperature', ({ atual: temperature }) => setTemp(`${(temperature / 10).toFixed(1)}°C`));

        return () => { 
            window.api.close('date'); 
            window.api.close('wifi');
            window.api.close('isBusy'); 
            window.api.close('isBusyBlue'); 
            window.api.close('temperature');
            window.api.close('hasNotification'); 
        }
    }, []);

    return (
        <Wrapper>
            <Left>
                <Logo />
                <Busy style={style1}/>
            </Left>
            <Center>
                <Temp>{temp}</Temp>
            </Center>
            <Right>
                <Wifi>{hasNotification && <Warning data={{ size: 20 }} /> }</Wifi>
                <Date>{date}</Date>
                <Time>{time}</Time>
                <Wifi>{renderWifi}</Wifi>
                {/* <Wifi /> */}
                <Busy style={style2}/>
            </Right>
        </Wrapper>
    );
}