const translate = window.language.translate

export function close(isMounted) {

    window.api.close('render'); 
    window.api.close('updateSlot'); 
    window.api.close('SOUND/Init');
    window.api.close('LANGUAGE/change'); 

    window.api.close('imageZoom'); 
    window.api.close('imageResult'); 
    window.api.close('imageProcesso'); 

    window.api.close('hasNotification'); 
    window.api.close('wifiConnected'); 

    window.api.close('temperature');
    isMounted = false;
}

export function render(data, { _renderScreen }) {
    const renderScreen = _renderScreen[data.mode][data.type];
    if (typeof(renderScreen) == 'function') renderScreen(data);
}

export function updateSlot(data, { _caps }) {
    
    data.forEach(({ reset, number, block, cor: color, resultado: result, tempoRestante, tempoTotal }) => {
        if(!reset) {
            let { percentage } = window.math.calculate({ remaining: tempoRestante, total: tempoTotal });
            if (percentage > 99) percentage = 99;

            _caps.color[number](color);
            _caps.percent[number](percentage);

            if (typeof result !== 'boolean') return _caps.state[number](`${percentage}%`);

            if (result == false) _caps.percent[number](100);
            if (percentage >= 99) _caps.percent[number](100);
            
            _caps.state[number](result);

        } else {
            _caps.state[number]('');
            _caps.percent[number](0);
            _caps.color[number]('white');
            
            if (block) {
                _caps.canOpen[number](false);
                _caps.canceled[number](true);
            }
        }
    })
}

export function imageProcesso(img, { _report, _render }) {
    const hold_processo = window.variables.get_hold_processo();
    if (hold_processo == undefined) return;

    // console.log(`debug imageProcesso: ${JSON.stringify(hold_processo)}`)

    const { 
        data: { 
            user, show,
            cavity,
            cycle, result, 
            batch, ampoule, 
            status, duration, 
            equip, autoclave, 
            type, date, code, 
            qrcode,
            slot: { 
                time, color, 
                counter, slot_result,
            }
        }
    } = hold_processo;

    let { percentage } = window.math.calculate(time);
    if (percentage > 99) percentage = 99;

    if (percentage >= 99) _report.setSlotPercentReport(100);

    (slot_result) 
        ? _report.setSlotStateReport(true) 
        : _report.setSlotStateReport(false);

    if (typeof slot_result !== 'boolean') _report.setSlotStateReport(`${percentage}%`);
    
    _report.setImageReport(img);
    _report.setDateReport(date);
    _report.setUserReport(user);
    _report.setCodeReport(code);
    _report.setLoteReport(batch);
    _report.setCycleReport(cycle);
    _report.setIncubReport(equip);
    _report.setCapsulaReport(show);
    _report.setEquipReport(autoclave);
    _report.setDurationReport(duration);
    _report.setTypeReport(translate(type));
    _report.setCavityReport(cavity.toString().padStart(2, 0));
    _report.setQrCodeReport(qrcode);
    _report.setStatusReport(translate(status));
    _report.setResultReport(translate(result));
    _report.setAmpolaReport(translate(ampoule));

    _report.setSlotReport(counter);
    _report.setSlotColorReport(color);
    _report.setSlotPercentReport(percentage);

    _render.enableReport(true);
}

export function imageResult(img, { _result, _render }) {
    const hold_result = window.variables.get_hold_result();
    if(hold_result == undefined) return;

    const { 
        data: { 
            type, 
            number,
            time,
            result,
            qrcode,
            slot: { 
                cor: color 
            }
        }
    } = hold_result;

    let { percentage, hours, minutes } = window.math.calculate(time);
    if (percentage > 99) percentage = 99;

    if (percentage >= 99) _result.setPercentResult(100);

    (result) 
        ? _result.setStateResult(true) 
        : _result.setStateResult(false);

    if (typeof result !== 'boolean') _result.setStateResult(`${percentage}%`);
    if (result == false) _result.setPercentResult(100);

    _result.setImageResult(img);
    _result.setTypeResult(type);
    _result.setColorResult(color);
    _result.setStateResult(result);
    _result.setNumberResult(number);
    _result.setCanceledResult(false);
    _result.setDetectedResult(result);
    _result.setQrCodeResult(qrcode);
    _result.setPercentResult(percentage);
    _result.setTimeResult(`${hours}H${minutes}`);

    _render.enableResult(true);
}

export function imageZoom(img, { _zoom, _render }) {

    const hold_zoom = window.variables.get_hold_zoom();
    if(hold_zoom == undefined) return

    const { 
        visible, 
        data: { 
            slot, 
            type, 
            image 
        }
    } = hold_zoom;

    if(!visible) return _render.enableZoom(visible);

    _zoom.setSlotZoom(slot);
    _zoom.setTypeZoom(type);

    if (typeof image !== 'number') {
        _zoom.setImageZoom(image);
        return _render.enableZoom(visible);
    }

    _zoom.setImageZoom(img);
    _render.enableZoom(visible);
}