import React, { useEffect } from 'react';
import Manager from '../../scripts/manager';
import Page1   from './components/pages/Page1';
import Page2   from './components/pages/Page2';
import Page3   from './components/pages/Page3';
import TopBar  from './components/unique/TopBar';
import { close, render, updateSlot } from './functions';
import { imageProcesso, imageResult, imageZoom } from './functions';

export default function App() {

	const EVENTS = [
		'LANGUAGE/change',
		'SOUND/Init',
		'render',
		'updateSlot',
		'imageZoom',
		'imageResult',
		'imageProcesso',
		'wifiConnected',
		'hasNotification',
		'temperature',
	]

	const _application = Manager();
	const { _render, _menu, _calibrate, _wifi } = _application;

    useEffect(() => {
		let isMounted = true;
		if(isMounted) {
			window.api.update(EVENTS[0], (data) => window.language.change(data));
			window.api.update(EVENTS[1], (data) => _menu.enableSound(data));
			window.api.update(EVENTS[2], (data) => render(data, _application));
			window.api.update(EVENTS[3], (data) => updateSlot(data, _application));
			window.api.update(EVENTS[4], (img)  => imageZoom(img, _application));
			window.api.update(EVENTS[5], (img)  => imageResult(img, _application));
			window.api.update(EVENTS[6], (img)  => imageProcesso(img, _application));
			window.api.update(EVENTS[7], (data) => _menu.setSelectedNetwork(data));
        	window.api.update(EVENTS[8], (data) => (!data) && _menu.enableNotification(false));
			window.api.update(EVENTS[9], ({ atual: temperature }) => _calibrate.setTemp(`${(temperature / 10).toFixed(1)}`));

			const value = window.io.read('wifi','isON', true);
			window.api.send('enable/dissable/wifi', value);
			_wifi.setSelected(value);
		}

		return () => close(isMounted);
    }, []);

	return (
		<>
			<TopBar />
			{_render['Temperature'] && <Page1 />}
			{_render['Loading'] 	&& <Page2 />}
			{_render['Slots'] 		&& <Page3 data={_application} />}
		</>
	)
}

