const fs = require("fs");
const path = require("path");
const Jimp = require('jimp');
const WLDConfig = require('./WLDConfig');
const nodeCamera = require('node-webcam');
const { execSync } = require('child_process');

const glConfig = WLDConfig;

const loCameraOpts = {
    delay: 0, 
    frames: 8,
    width: 480, 
    height: 640,
    quality: 100, 
    output: "jpeg", 
    saveShots: true,
    device: `/dev/video0`,
    callbackReturn: "location", 
    verbose: false
}

const FILE = {
    base:  '/home/pi/wld/pics/',
    temp:  '/home/pi/wld/pics/temp/temp.jpg',
    blue:  '/home/pi/wld/pics/temp/azul.jpg',
    black: '/home/pi/wld/pics/common/black.jpg',
}

const glCamera = nodeCamera.create(loCameraOpts);

class WLDCamera {
    constructor() {
        this.loop;
        this.media;
        this.status;
        this.percent;
        this.glMedia; //Novo Algoritimo
        this.list = [];
        this.tarefa = 0;
        this.vezesTeste;
        this.currentSlot;
        this.tipoCorte = 370;

        this.CONFIG = {
            numberOfTrys: 5,
            ignore: glConfig.readInt('settings', 'ignore', 1),
            media_corte: (this.tipoCorte == 370) 
                ? glConfig.readInt('settings', 'media_corte37', 20) 
                : glConfig.readInt('settings', 'media_corte60', 10),
        }

        this.CTOK = 1;
        this.CTERRO = -1;
        this.CTPROCESSANDO = 0;

        this.WLDLog = (type, value) => console.log(`[${type}] -> ${value}`);
    }

    isCameraConected = () => execSync('ls /dev/video*').toString().includes('video0');

    begin() {
        this.loop = setInterval(async () => {
            switch (this.tarefa) {
                case 0:
                break
                case 1:
                    if (this.vezesTeste == 0) {
                        this.tarefa = 0;
                        this.status = this.CTERRO;
                        break
                    }

                    this.vezesTeste--;

                    if(this.isCameraConected()) {
                        this.tarefa = 0;
                        this.status = this.CTOK;
                        this.WLDLog('CAMERA',`Camera Found`);
                        break
                    }
                break
                case 2:
                    if (this.vezesTeste == 0) {
                        this.tarefa = 0;
                        this.status = this.CTERRO;
                        break
                    }

                    this.tarefa = 4;
                    try {
                        await fs.promises.access(FILE['temp'], fs.constants.F_OK);
                        
                        this.vezesTeste = this.CONFIG['numberOfTrys'];

                        const data = await this.turnPhotoToBlue();
                        if (data) this.tarefa = data;

                    } catch (error) {
                        if(err) {
                            console.log(`[CAMERA] -> ERROR TEMPORARY FILE: 1, NOT FOUND`);
                            console.log(`[CAMERA] -> ${err}`);
                        }
                        this.vezesTeste--
                        this.tarefa = 2
                    }
                break
                case 3:
                    if (this.vezesTeste == 0) {
                        this.tarefa = 0;
                        this.status = this.CTERRO;
                        console.log('[CAMERA] -> ERROR BLUE FILE NOT FOUND');
                        break
                    }
                        
                    this.tarefa = 0;

                    try {
                        await fs.promises.access(FILE['blue'], fs.constants.F_OK);
                        this.media = this.glMedia;

                        if (this.media >= this.currentSlot['media']) {
                            this.currentSlot['media'] = this.media;

                            if (this.currentSlot['media'] > this.currentSlot['media_corte']) {
                                this.movePhoto(true);
                                this.WLDLog('CAMERA', 'Blue Photo has Been Moved');
                            } else {
                                this.movePhoto(false);
                                this.WLDLog('CAMERA', 'Black Photo has Been Moved');
                            }
                        }

                        this.status = this.CTOK;
                    } catch (err) {
                        this.tarefa = 3;
                        this.vezesTeste--;
                        console.log(`[CAMERA] -> ERROR BLUE FILE NOT FOUND`);
                        console.log(`[CAMERA] -> ${err}`);
                    }
                break
            }
        }, 100);
    }

    close() {
        this.tarefa = 0;
        this.loop = null;
        this.status = null;
        this.vezesTeste = null;
        clearInterval(this.loop);
    }

    statusFoto = () => this.status;
    statusCamera = () => this.status;

    startTesteCamera() {
        this.begin();
        this.status = this.CTPROCESSANDO;
        this.vezesTeste = this.CONFIG['numberOfTrys'];
        this.tarefa = 1;
    }

    takePhoto(paCapsula) {
        this.currentSlot = paCapsula;
        this.status = this.CTPROCESSANDO;
        this.currentSlot['fotoLiberada'] = false;

        glCamera.capture(FILE['temp'], (err) => {
            if (err) {
                this.tarefa = 0;
                this.status = this.CTERRO;
                this.WLDLog('CAMERA',`ERROR WHILE TAKING PHOTO`);
                return
            }

            this.tarefa = 2;
            this.WLDLog('CAMERA',`Photo Taken Succesfully`);
        })
    }

    async convertPhoto(file) {
        try {
            const molde = await Jimp.read(path.join(__dirname, '../../assets/print/molde.png'));
            const image = await Jimp.read(`/home/pi/wld/pics/ensaios/${file}.jpg`);

            const { width, height, data } = image['bitmap'];

            image.scan(0, 0, width, height, (x, y, idx) => {
                if (data[idx + 2] > 200) {
                    data[idx + 2] = 0;
                } else {
                    data[idx + 0] = 255;
                    data[idx + 1] = 255;
                    data[idx + 2] = 255;
                }
            })

            image.resize(378,378);
            image.blit(molde,0,0);

            image.write(`/home/pi/wld/pics/print/${file}.png`);
        } catch (e) {
            this.WLDLog('CAMERA',`ERROR WHILE CONVERTING PHOTO FOR PRINTING`);
            this.WLDLog('CAMERA', e.message);
        }
    }

    async turnPhotoToBlue() {
        try {
            const image = await Jimp.read(FILE['temp']);

            const { width, height, data } = image['bitmap'];

            let loAzul = 0;
            let loTotal = 0;

            image.scan(0, 0, width, height, (x, y, idx) => {
                data[idx + 0] = 0;
                data[idx + 1] = 0;

                loAzul += data[idx + 2];
                loTotal++;  

                (data[idx + 2] > this.currentSlot['media_corte'])
                    ? data[idx + 2] = 255
                    : data[idx + 2] = 0;

            })

            this.glMedia = loAzul / loTotal;

            this.WLDLog('CAMERA', `media: ${this.glMedia.toFixed(2)}.`);

            if(this.currentSlot['etapa'] == 0 || 
                (this.currentSlot['etapa'] == 1 && 
                this.currentSlot['media_corte'] < this.glMedia + this.CONFIG['media_corte'])) { // Determina Media de corte com primeira leitura

                this.WLDLog('CAMERA',`media de corte no arquivo ini: ${this.CONFIG['media_corte'].toFixed(2)}.`)
                this.currentSlot['media_corte'] = this.glMedia + this.CONFIG['media_corte'];
                this.WLDLog('CAMERA',`etapa: ${this.currentSlot['etapa']}.`);
                this.WLDLog('CAMERA',`media de corte no arquivo ini + primeira leitura: ${this.currentSlot['media_corte'].toFixed(2)}.`);
                this.currentSlot['glDebug'].write(`media_corte${this.currentSlot['etapa']+1}`, this.currentSlot['media_corte'].toFixed(2));
            }
            
            this.currentSlot['glDebug'].write(this.currentSlot['etapa'] + 1, this.glMedia.toFixed(2));
            this.currentSlot['etapa']++;

            if(this.currentSlot['ignora'] > 0) {
                this.WLDLog('CAMERA',`Photo Ignored`);
                this.currentSlot['ignora']--;
                this.glMedia = 0;
            }

            image.write(FILE['blue']);
            return 3;
        } catch (e) {
            this.WLDLog('CAMERA','ERROR WHILE TURNING PHOTO TO BLUE');
            this.WLDLog('CAMERA', e.message);
            return 2;
        }
    }

    movePhoto(paFlag) {
        if (paFlag) {
            fs.rename(FILE['blue'],`${FILE['base']}ensaios/${this.currentSlot['idIncubacao']}.jpg`, err => err && this.WLDLog('CAMERA', `erro ao mover a foto: ${err}`))
            return
        }

        this.copyPhoto(FILE['black'], `${FILE['base']}ensaios/${this.currentSlot['idIncubacao']}.jpg`);
    }

    copyPhoto(oldPath, newPath) {
        var readStream = fs.createReadStream(oldPath);
        var writeStream = fs.createWriteStream(newPath);

        readStream.on('error', (err) => {
            this.WLDLog('CAMERA','ERROR WHILE READ PHOTO FILE');
            this.WLDLog('CAMERA', err);
        });

        writeStream.on('error', (err) => {
            this.WLDLog('CAMERA','ERROR WHILE COPYING PHOTO FILE');
            this.WLDLog('CAMERA', err);
        });

        readStream.pipe(writeStream);
    }

    setMediaCorte(value) {
        this.tipoCorte = value;
        this.CONFIG.media_corte = (this.tipoCorte == 370) 
            ? glConfig.readInt('settings', 'media_corte37', 20) 
            : glConfig.readInt('settings', 'media_corte60', 10);
    }

}

module.exports = WLDCamera;