const fs = require('fs');
const path = require("path");

const { 
    cpSync, 
    mkdirSync, 
    accessSync, 
    unlinkSync,
    readdirSync, 
    copyFileSync, 
    appendFileSync, 
    constants: { F_OK }, 
} = require('fs');
const { execSync } = require('child_process');

class WLDIO {

    async verifyFolder(folder) {
		try {
			accessSync(folder, F_OK);
			return true;
		} catch (err) {
			return false;
		}
	}

    async createFolder(folder) {
		try {
			mkdirSync(folder);
			return true;
		} catch (err) {
			return false;
		}
	}

    async writeFile({ filePath, outData }) {
		try {
			appendFileSync(filePath, outData, 'utf8');
			return true;
		} catch (err) {
			return false;
		}
	}

    async listDir(folderPath) {
        try {
            const response = readdirSync(folderPath);
            return response;
        } catch (err) {
            return false;
        }
	}

    async verifyPendrive() {
        try {
            const response = execSync('ls -la /media/pi/').toString();

            const lines = response.split('\n');
            
            let final_split;

            for (const line of lines) {
                if (line[0] == 'd' && !line.includes('root')) final_split = line.split(' ');
            }

            return final_split[final_split.length - 1]
        } catch (err) {
            return false;
        }
	}

    async copyFile(source, output) {
        try {
            copyFileSync(source, output);
            return true;
        } catch (error) {
            return false;
        }
	}

    async copyFolder(source, output) {
        try {
            cpSync(source, output, { recursive: true });
            return true;
        } catch (error) {
            return false;
        }
	}

    async writeJson(obj, output) {
        const writing = JSON.stringify(obj, null, 4);

        try { 
            fs.writeFileSync(output, writing); 
            return true;
        } catch (error) {
            return false;
        }
    }

    readVersion(file) {
        try {
			return JSON.parse(fs.readFileSync(file));
		} catch (error) {
            this.writeJson({ version: 0 }, file);
			return { version: 0 };
		}
    }

    RemoveDotFiles = () => {
		const directory_pics   = "/home/pi/wld/pics";
		const directory_temp   = "/home/pi/wld/pics/temp";
		const directory_print  = "/home/pi/wld/pics/print";
		const directory_common = "/home/pi/wld/pics/common";
		const directory_essays = "/home/pi/wld/pics/ensaios";

		const _pics    = readdirSync(directory_pics);
		const _temps   = readdirSync(directory_temp);
		const _prints  = readdirSync(directory_print);
		const _commons = readdirSync(directory_common);
		const _essays  = readdirSync(directory_essays);

		for (const _pic    of _pics)    { if(_pic[0]    == '.') unlinkSync(path.join(directory_pics,   _pic));    }
		for (const _temp   of _temps)   { if(_temp[0]   == '.') unlinkSync(path.join(directory_temp,   _temp));   }
		for (const _print  of _prints)  { if(_print[0]  == '.') unlinkSync(path.join(directory_print,  _print));  }
		for (const _common of _commons) { if(_common[0] == '.') unlinkSync(path.join(directory_common, _common)); }
		for (const _essay  of _essays)  { if(_essay[0]  == '.') unlinkSync(path.join(directory_essays, _essay));  }

	}

}

module.exports = new WLDIO();