const fs = require('fs');
const path = require('path');
// const os = require('os');
const os = require('os');
// const si = require('systeminformation');
const { execSync } = require('child_process');
const QRCode = require('qrcode');

const WLDIO = require('./WLDIO');
const { WLD } = require('./WLDUtils');
const WLDWifi = require('./WLDWifi');

const WLDSerial = require('./WLDSerial');
const WLDUpdate = require('./WLDUpdate');

const WLDNotifications = require('./WLDNotifications');

const WLDHL       = require('./WLDHL');
const WLDCapsula  = require('./WLDSlot');
const WLDTimer    = require('./WLDTimer');
const WLDCamera   = require('./WLDCamera');
const WLDConfig   = require('./WLDConfig');
const WLDPrinter  = require('./WLDPrinter');
const WLDBD       = require('./WLDDatabase');
const { translate, setCurrent } = require('./WLDLanguage');


const BackupController = require('../controllers/BackupController');
const UpdateController = require('../controllers/UpdateController');

const { renderer, emit } = require('./WLDWindow');
const { Alerta } = require('./WLDAlert');

const glBD         = WLDBD;
const glTempo      = new WLDTimer();
const glTempoPrint = new WLDTimer();
const glCamera     = new WLDCamera();
const glConf       = WLDConfig;
const glPrinter    = new WLDPrinter();

const STATE = {
	CAPSULE: {
		LOOP: 4,
		TEST: 5,
		INCREMENT: 6,
	},
	VERIFY: {
		PHOTO: 13,
		CAMERA: 10,
		HASPHOTO: 14,
		TEMPERATURE: 15,
	},
	SEND: {
		USB: 7,
		LED: 11,
		STATUS: 3,
		SELECTION: 1,
		DATE: { ASK: 26, WRITE: 29 },
	},
	WAIT: {
		RESET: 31,
		BUZZER: 28,
		SCREEN: 20,
		SELECTION: 0,
		TEMPERATURE: 2,
		LED: { ON: 12, OFF: 19 },
		DATE: { ASK: 27, WRITE: 30 },
		TIMER: { PHOTO: 21, CAMERA: 17 },
		USB: { ON: 8, OFF: 18, CANCEL: 32 },
	},

}

class WLDMain {
	constructor () {
		this.NBips;
		this.ct = 0;
		this.newDate;
		this.loops = [];
		this.NRESET = 5;
		this.currentkey;
		this.tarefa = STATE.SEND.DATE.ASK;
		this.datahora = '';
		this.apontador = 0;
		
		this.flagBip 	   = false;
		this.flagDate      = false;
		this.flagBipON     = false;
		this.flagNReset    = false;

		this.NUMERODECAPSULAS = 8;

		this.month = glConf.read('backup', 'month', '01');

		this.temperatura = { atual: 0 };

		this.Capsula = [
			new WLDCapsula(1), new WLDCapsula(2),
			new WLDCapsula(3), new WLDCapsula(4), new WLDCapsula(5), 
			new WLDCapsula(6), new WLDCapsula(7), new WLDCapsula(8),
		];

		this.capsulaStatus = [
			{ reset: true, number: 0 }, { reset: true, number: 1 },
			{ reset: true, number: 2 }, { reset: true, number: 3 },
			{ reset: true, number: 4 }, { reset: true, number: 5 },
			{ reset: true, number: 6 }, { reset: true, number: 7 },
		];

		
		this.info = {
			model: glConf.read('info','model', 'one'),
			serial_number: glConf.read('info','serial_number', '0000'),
			fabrication_date: glConf.read('info','fabrication_date', 'one'),
			version_software: `s${WLDIO.readVersion('/home/pi/wld/lummina/src/version.json')?.version}`,
			version_firmware: glConf.read('info','version_firmware', 'f202212010001'),
		};

		this.alert_types = {
			20: 'wifi',
			11: 'edit',
			12: 'camera',
			13: 'printer', 
			14: 'temperature', 
			15: 'pdf', 
			16: 'clearDatabase',
		};

		this.alert_functions = {
			20: () => renderer().close('popup', 'alerta'),
			11: () => this.getKey('date'),
			12: () => UpdateController.Download(),
			13: () => {}, 
			14: () => this.resetaTemperatura(), 
			15: 'pdf', 
			16: () => BackupController.clearDatabaseConfirm(),
		};

		this.status_types = {
			0: 51, // em andamento
			1: 103, // concluido
			2: 103, // concluido
			3: 52, // cancelado
		};

		this.result_types = {
			0: 137,
			1: 54,
			2: 55,
			3: 73,
		};

		this.code_types = {
			'clear':     () => BackupController.clearDatabase(),
			'import':    () => BackupController.importDatabase(),
			'date':      () => renderer().open('settings', 'date', { current: { ...WLD.getTime(), ...WLD.getDate()} }),
			'calibrate': () => this.calibrate(),
		};
		
		glBD.correctDatabase().then(() => WLD.log('BANCO DE DADOS', 'verificação no banco de dados OK'));
	}

	// #region PRINT

	async printerTesting() {
		const data = await glPrinter.isConected();
		if(data) return setTimeout(() => glPrinter.testPrinter({ number: this.info['serial_number'], date: WLD.getDate('dd/mm/yyyy'), hour: WLD.getTime('hh:mm') }), 1000);

		Alerta('printer', translate(62), translate(26), null, translate(63));
		return WLD.log('PRINT', 'Não foi possível imprimir, impressora não esta conectada.');
	}
	
	addPrint = (data) => glBD.insertNewData('prints', data);

	async checkPrint() {
		if(!glPrinter.readPrint()) {
			const result = await glBD.getData('prints');

			if(result?.id_essay) {
				const { id_essay } = result;
				await glBD.deleteQueue(id_essay);
				glPrinter.setPrint();
				this.print(id_essay);
			}
		}
	}
	
	async print(id) {
		
		const ensaio = await glBD.getData('default', id);

		const stringQR = await this.getQrCodeString(id);
		
		const resQR = await this.saveQRCode(stringQR);
		
		if(ensaio) {
			const _print = WLD.factory('print', ensaio);
			
			if(_print.image == '../common/cancelado') {
				const data = await glPrinter.isConected();
				
				if(data) return setTimeout(() => glPrinter.print(_print, resQR), 1000);

				glPrinter.freePrint();

				Alerta('printer', translate(62), translate(26), null, translate(63));
				return WLD.log('PRINT', 'Não foi possível imprimir, impressora não esta conectada.');
			}

			const data = await glPrinter.isConected();

			if(data) {
				await glCamera.convertPhoto(_print.image, this.Capsula[_print.cavidade]);
				return setTimeout(() => glPrinter.print(_print, resQR), 1000);
			}
			
			glPrinter.freePrint();
			Alerta('printer', translate(62), translate(26), null, translate(63));
			WLD.log('PRINT', 'Não foi possível imprimir, impressora não esta conectada.');
		}
	}

	// #endregion PRINT
	
	// #region DATE

	datedate(date) {
		this.flagDate = true;
		this.newDate = date;

		const directory = "/home/pi/wld/pics/ensaios";
		const directory2 = "/home/pi/wld/pics/print";

		BackupController.flagCanBackup = true;

		const ensaios = fs.readdirSync(directory);
		for (const ensaio of ensaios) { fs.unlinkSync(path.join(directory, ensaio)); }

		const files = fs.readdirSync(directory2);
		for (const file of files) { fs.unlinkSync(path.join(directory2, file)); }

		WLDIO.copyFile(`${WLDBD.clean_database_location}`,`${WLDBD.database_location}`);
		this.changeDate(date);
	}

	changeDate(date) {
		const day     = date.substring(6,   8);
		const year    = date.substring(0,   4);
		const hours   = date.substring(8,  10);
		const month   = date.substring(4,   6);
		const minutes = date.substring(10, 12);
		const seconds = date.substring(12, 14);

		BackupController.flagCanBackup = true;

		BackupController.today = day;

		const updateDate = `${year}${month}${day} ${hours}:${minutes}:${seconds}`;

		if (os.platform() !== 'darwin') {
			try { execSync(`sudo date --set="${updateDate}"`, { stdio : 'pipe' }); } 
			catch (error) { console.error(error); }
		}

		glTempoPrint.setTimer(2000);
	}

	monthHasChanged() {
		const date = new Date();

		let month = (date.getMonth() + 1).toString().padStart(2, 0);

		if(this.month !== month) {
			this.month = month;
			glConf.write('backup', 'month', month);
			return true;
		}

		return false;
	}

	// #endregion DATE

	// #region closeAlerta
	closeAlerta = (type) => renderer().close('popup', 'alerta', { type: this.alert_types[type] });
	// #endregion

	//#region alertConfirmPressed
	alertConfirmPressed(type) {
		const command = this.alert_functions[type];
		if(command) command();
		this.closeAlerta(type);
	}
	//#endregion

	// #region INDEFINIDO

	verifyKey(params) {

		const { data: { digits, type } } = params;
		renderer(params);
		
		const verify = WLDHL.keyVerify(this.currentkey);
		if(digits !== verify) return Alerta('pdf', translate(100), translate(26), null, translate(11)); 
		const command = this.code_types[type];
		if(command) command();
	}

	getKey(type) {

		const estadoatual = WLDSerial.leEstado();

		if (estadoatual !== '00000000') {
			Alerta('temperature', translate(135),translate(26), null, translate(11));
			return WLD.log('EXPORT', 'não é possível realizar essa ação com testes em andamento.');
		}

		const { pwd } = WLDHL.getCharCodes(this.info['serial_number'])

		this.currentkey = WLDHL.key(pwd);
		renderer().open('settings', 'key', { key: this.currentkey, type });
	}

	sendOffset() {
		this.temperatura.offset = glConf.read('calibrate', 'temperature_offset', 0);
		WLDSerial.sendSerial(`*T${parseInt(this.temperatura?.desejada) - this.temperatura.offset * 10}#`);
	}

	calibrate = () => {

		const estadoatual = WLDSerial.leEstado();

		if (estadoatual !== '00000000') {
			Alerta('temperature', translate(135), translate(26), null, translate(11));
			return WLD.log('EXPORT', 'não é possível realizar essa ação com testes em andamento.');
		}

		renderer().open('settings', 'calibrate', { offset: glConf.read('calibrate', 'temperature_offset', 0) });
	}

	verifyLanguage = () => {

		const estadoatual = WLDSerial.leEstado();

		if (estadoatual !== '00000000') {
			Alerta('temperature', translate(135), translate(26), null, translate(11));
			return WLD.log('EXPORT', 'não é possível realizar essa ação com testes em andamento.');
		}

		renderer().open('settings', 'language');

	} 
	
	setLanguage({ selected }) {
		setCurrent(selected);
		emit('LANGUAGE/change', selected);
		renderer().close('settings', 'language');
	}

	async openReport({ id, index }) {
		const ensaio = await glBD.getData('default', id);
		if(ensaio) this.senderResult(ensaio, index, true, false);
	}

	initSound() {
		let bipIni = glConf.read('settings','can_bip', true);
		emit('SOUND/Init', bipIni);
	}

	changeSound = (value) => glConf.write('settings', 'can_bip', value);
	printQRcode = (value) => glConf.write('settings', 'print_qrcode', value);


//#region clickOnSlot
	async clickOnSlot({ index }) {

		const tarefa = this.Capsula[index].tarefa;
		const id = this.Capsula[index].idIncubacao;

		switch (tarefa) {
			case 2:
				const ensaio = await glBD.getData('default', id);
				if (ensaio) this.senderResult(ensaio, index);
			break
			case 3:
				const ensaios = await glBD.getData('default', id);
				if(ensaios) this.senderResult(ensaios, index, true);
			break
			default:
				const database = { user: await glBD.getDatas('users'), equip: await glBD.getDatas('equips') }

				const ampoule = glConf.readInt('default', 'ampoule', 71);
				const type60  = glConf.readInt('default', 'type60', 119);
				const type37  = glConf.readInt('default', 'type37', 2);

				const _data = {
					database, slot: index,
					default: { type60, type37, ampoule },
					type: (this.temperatura.desejada == 370) ? false : true,
				}
				
				renderer().open('popup', 'start', _data);
			break
		}
	}
//#endregion

//#region map
	map(object, callback) {
		return Object.keys(object).reduce((result, key) => {
		  result[key] = callback(object[key]);
		  return result;
		}, {});
	}

	delay(ms) {
		return new Promise(resolve => setTimeout(resolve, ms));
	}
//#endregion
//#region startTeste
	async startTeste(args) {

		const { options, cavity, type } = args;
		
		renderer().close('popup', 'start');

		args.date = `${WLD.getDate('dd/mm/yyyy')} - ${WLD.getTime('hh:mm:ss')}`; 

		let query = await glBD.getData('essays');
		let loIndex;

		(query == undefined)
			? loIndex = 0
			: loIndex = query.id_essay;
		
		loIndex++;

        await glBD.insertNewData('essays', args);
		
		const key = (type === 2 || type === 203) ? 'type37' : 'type60';
		
		glConf.write('default', key, type);

		if(loIndex) {
			const qrcode = await this.generateQrCode(loIndex);

			renderer().open('popup', 'wait', { slot: cavity, qrcode });

			WLD.log('BANCO DE DADOS',`Novo Registro: ${loIndex}`);

			this.Capsula[cavity].tarefa = 1;
			this.Capsula[cavity].tipo = type;
			this.Capsula[cavity].cor = options.color;
			this.Capsula[cavity].idIncubacao = loIndex;
			this.Capsula[cavity].tempo = (options.time * 60 * 1000);
		}
	}
//#endregion

	contadorCapsula() {
		for(const capsula of this.capsulaStatus) {
			capsula.ct = this.ct++ ;
			this.ct &= 0xff;
		}

		emit('updateSlot', this.capsulaStatus);
		setTimeout(() => this.contadorCapsula(), 5000);
	}

	async senderResult(essay, index, key = false, show = true) {

		const _data = {
			show,
			date:    essay['date'],
			type:    essay['type'],
			user:    essay['name'],
			code:    essay['code'],
			batch:   essay['batch'],
			cycle:   essay['cycle'],
			ampoule: essay['ampoule'],
			equip:   essay['id_essay'],
			cavity:  (essay['cavity'] + 1).toString().padStart(2, '0'),

			autoclave: (essay['model'] == 'CONTROLE') 
				? '' 
				: essay['model'],

			duration: (key) 
				? (essay['duration'] == 0) 
					? '00:00:00'
					: essay['duration'] 
				: WLD.convSegHHMMSS(this.Capsula[index].tempoDecorrido),

			photo: (key) 
				? essay['id_essay'] 
				: (this.Capsula[index].liberaMostra()) ? essay['id_essay'] : 'common/black',

			slot: {
				counter:     this.Capsula[index].cv,
				color:       this.Capsula[index].cor,
				slot_result: this.Capsula[index].resultado,
				time: {
					total:     this.Capsula[index].tempo,
					remaining: this.Capsula[index].ctTempo,
				},
			},
			qrcode: await this.generateQrCode(essay['id_essay']),

		}

		_data.status = this.status_types[essay['result']];
		_data.result = this.result_types[essay['result']];

		renderer().open('screen', 'processo', _data);

		return true;
	}

	saveQRCode(string, commands) {

		const qrcode_path = '/home/pi/wld/pics/qrcode.png';

		const options = { 
			width: commands?.width || 350,
			height: commands?.width || 350
		};

		function handle(err, resolve) {
			if (err) throw err;
			// WLD.log('MAIN', `success`);
			resolve(true);
		}

		return new Promise((resolve, reject) => {
			QRCode.toFile(qrcode_path, string, options, (err) => handle(err, resolve));
		});
	}

	async getQrCodeString(id) {
		let qrcode_data = await glBD.getData('qrcode', id);
		
		if(!qrcode_data) qrcode_data = await glBD.getData('qrcode', id);

		let text = `${this.info.serial_number}#${this.info.version_software}#`;

		for(var key in qrcode_data) {
			if(key == 'type') { 
				text += translate(qrcode_data[key]) + '#';
				continue;
			}
			
			if(key == 'result') { 
				switch (qrcode_data[key]) {
					case 0: // 0 em andamento 51
						text += translate(51) + '#';
					break;
					case 1: // 1 positivo 54
						text += translate(54) + '#';
					break;
					case 2: // 2 negativo 55
						text += translate(55) + '#';
					break;
					case 3: // 3 cancelado 52
						text += translate(52) + '#';
					break;
				}
				continue;
			}

			if(key == 'cavity') {
				text += qrcode_data[key] + 1;
				continue;
			}

			text += qrcode_data[key] + '#';
		}
		
		if((text.match(/#/g) || []).length != 11) return 'erro';

		return text;
	}

	async generateQrCode(id) {
		const text = await this.getQrCodeString(id);

		if(text != 'erro') {
			const qrcode = await QRCode.toDataURL(text, { color: { dark: "#FFFFFFFF", light: "FFFFFF00" }});
			return qrcode;
		}

		return 'erro';
	}

	resetaTemperatura() {
		this.tarefa = 0;
		delete this.temperatura['desejada'];
		renderer({ mode: 'screen', type: 'temperature' });
	}

	async openEssays() {
		const data = await glBD.getDatas('all_essays');
		if (data) renderer().open('popup', 'essays', data);
	}

	openConfig(args) {
		args.data = { 
			info: { 
				model: this.info['model'],
				serial_number: this.info['serial_number'],
				version_software: this.info['version_software'], 
				version_firmware: this.info['version_firmware'], 
				fabrication_date: this.info['fabrication_date'], 
			},
			print_qr: glConf.read('settings', 'print_qrcode', false)
		}

		renderer(args);
	}

	async cancelaEnsaio(type, index, block) {
		this.Capsula[index].encerraEnsaio()
		this.Capsula[index].tarefa = (type) ? 3 : 0;

		const edit = {
			result: { 
				_old: 'result', 
				_type: 'id_essay',
				_id: this.Capsula[index].idIncubacao,
				_new: (!type) ? 3 : (this.Capsula[index].resultado) ? 1 : 2,
			},
			duration: { 
				_old: 'duration',
				_type: 'id_essay',
				_id: this.Capsula[index].idIncubacao,
				_new: WLD.convSegHHMMSS(this.Capsula[index].tempoDecorrido),
			} 
		}

		if (!type) {
			this.capsulaStatus[index].reset = true
			this.capsulaStatus[index].resultado = 0
		}

		if (block) this.capsulaStatus[index].block = true;

		await glBD.editData('essays', edit['result']);
		await glBD.editData('essays', edit['duration']);

		const results = {
			number: index + 1,
			type:   this.Capsula[index].tipo,
			slot:   this.capsulaStatus[index],
			image:  this.Capsula[index].idIncubacao,
			result: this.Capsula[index].resultado,
			qrcode: await this.generateQrCode(this.Capsula[index].idIncubacao),
			time: {
				total: 	   this.Capsula[index].tempo,
				remaining: this.Capsula[index].ctTempo,
			},
		}

		this.flagBip = true;
		this.NBips = glConf.read('settings', 'number_of_bips', 5);
		this.flagBipON = glConf.read('settings', 'can_bip', true);

		renderer().open('popup', (type) ? 'result' : 'cancel', results);
		glBD.insertNewData('prints', { id_essay: this.Capsula[index].idIncubacao });
	}

	closeResult(args) {
		this.flagBip = true;
		this.NBips = 0;
		renderer(args);
	}

	updateDate() {
		emit('date', { date: WLD.getDate('dd/mm/yyyy'), time: WLD.getTime('hh:mm') });
		setTimeout(() => this.updateDate(), 1000);
	}

	// #endregion INDEFINIDO

	// #region MAIN LOOP 

	async loop() {
		switch (this.tarefa) {
			case STATE.SEND.DATE.ASK:
				WLDSerial.sendSerial(`*D?#`);
				this.tarefa = STATE.WAIT.DATE.ASK;
			break
			case STATE.WAIT.DATE.ASK:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.changeDate(WLDSerial.retorno);
						this.tarefa = STATE.WAIT.SELECTION;
					break
					case WLDSerial.CTERRO:
						WLD.log('SERIAL', `erro não teve resposta no comando: *D?#`);
						this.tarefa = STATE.WAIT.SELECTION;
					break
				}
			break
			case STATE.WAIT.SELECTION:
				if(this.temperatura?.desejada) {
					renderer({ mode: 'screen', type: 'loading' });
					this.tarefa = STATE.SEND.SELECTION;
					glCamera.setMediaCorte(this.temperatura?.desejada);
				}
			break
			case STATE.SEND.SELECTION:
				this.sendOffset();
				this.tarefa = STATE.WAIT.TEMPERATURE;
			break
			case STATE.WAIT.TEMPERATURE:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						emit('temperature', { atual: WLDSerial.retorno })
						this.temperatura.atual = WLDSerial.retorno

						let x = WLDSerial.retorno;
						let y = this.temperatura?.desejada;

						if (x >= (y - 30) && x <= (y + 30)) {
							glTempo.setTimer(3000);
							this.tarefa = STATE.WAIT.SCREEN;
						} else {
							glTempo.setTimer(2000);
							this.tarefa = STATE.SEND.SELECTION;
						}
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', `erro não teve resposta no comando: *T${this.temperatura?.desejada}#`)
						this.tarefa = STATE.SEND.SELECTION;
					break
				}
			break
			case STATE.WAIT.SCREEN:
				renderer({ mode: 'screen', type: 'slots' });
				this.initSound();
				if (glTempo.state()) this.tarefa = STATE.SEND.STATUS;
			break
			case STATE.SEND.STATUS:
				WLDSerial.sendSerial(`*S#`);
				this.tarefa = STATE.CAPSULE.LOOP;
			break
			case STATE.CAPSULE.LOOP:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						for (let index = 0; index < this.NUMERODECAPSULAS; index++) {
							// if(this.capsulaStatus[index]['block'] == undefined) { # Retirado 2022/11/17 15:39:49
								if (WLDSerial.retorno[index] == '1') {
									this.Capsula[index].estado = true

									switch (this.Capsula[index].tarefa) {
										case 0: // capsula colocada indevidamente -> tarefa 4
											this.Capsula[index].tarefa = 4;
										break
										case 1: // inicia o teste -> tarefa 2
											this.Capsula[index].startEnsaio(); // Seta valores de inicio de teste de capsula
											this.Capsula[index].tarefa = 2;
											this.capsulaStatus[index].reset = false;
											renderer({ mode: 'popup', type: 'none' });
										break
										case 2: // verifica se teste encerrou -> tarefa 3
											if(this.Capsula[index].testaEnsaio()) this.cancelaEnsaio(true, index);
										break
										case 4: // mostra erro
											renderer({ mode: 'popup', type: 'error', data: { slot: index, qrcode: 'erro' }});
										break
									}

									//atualizar porcentagem e tempo

									this.capsulaStatus[index].number         = index;
									this.capsulaStatus[index].cor            = this.Capsula[index].cor;
									this.capsulaStatus[index].tempoTotal     = this.Capsula[index].tempo;
									this.capsulaStatus[index].resultado      = this.Capsula[index].resultado;
									this.capsulaStatus[index].tempoRestante  = this.Capsula[index].ctTempo;;
									this.capsulaStatus[index].tempoDecorrido = this.Capsula[index].tempoDecorrido;

								} else {
									switch (this.Capsula[index].tarefa) {
										case 2: // capsula tirada antes do tempo, teste interrompido -> tarefa 0
											this.cancelaEnsaio(false, index);
										break
										case 3: // reseta ao terminar teste e tirar capsula -> tarefa 0
											this.capsulaStatus[index].reset = true;
											this.Capsula[index].tarefa = 0;
										break
										case 4: // tira popup de erro, e verifica todas as capsulas se estão para iniciar teste -> tarefa 0
											renderer({ mode: 'popup', type: 'none' });

											for(let index2 = 0; index2 < this.NUMERODECAPSULAS; index2++) {
												const task = this.Capsula[index2].tarefa;

												if(task == 4) this.Capsula[index2].tarefa = 0;

												if(task == 1) renderer({ mode: 'popup', type: 'wait', data: { slot: index2 }});
											}
										break
										default:
											this.Capsula[index].estado = false;
											this.capsulaStatus[index].reset = true;
											this.capsulaStatus[index].number = index;
										break
									}
								}
							// }
						}

						this.tarefa = STATE.CAPSULE.TEST;
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', 'Erro não teve respota no comando: *S#')
						this.tarefa = STATE.CAPSULE.TEST;
					break
					case WLDSerial.CTERRORECEBE: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', 'Erro na recepção: *S#')
						this.tarefa = STATE.CAPSULE.TEST;
					break
				}
			break
			case STATE.CAPSULE.TEST:
				this.tarefa = STATE.CAPSULE.INCREMENT;
				if (!this.Capsula[this.apontador].fim) {
					if (this.Capsula[this.apontador].intervalo == 0) {
						this.Capsula[this.apontador].intervalo = this.Capsula[this.apontador].CTINTERVALO
						this.tarefa = STATE.SEND.USB;
					}
				}
			break
			case STATE.CAPSULE.INCREMENT:
				this.apontador++
				if (this.apontador == this.NUMERODECAPSULAS) this.apontador = 0
				this.temperatura.offset = glConf.read('calibrate', 'temperature_offset', 0);
				this.sendOffset();
				this.tarefa = STATE.VERIFY.TEMPERATURE;
			break
			case STATE.SEND.USB:
				WLDSerial.sendSerial(`*U${this.apontador + 1}#`)
				WLD.log('SERIAL', `*U${this.apontador + 1}# OK`)
				this.tarefa = STATE.WAIT.USB.ON;
			break
			case STATE.WAIT.USB.ON:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.tarefa = STATE.WAIT.TIMER.CAMERA;
						glTempo.setTimer(2000);
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', `erro não teve resposta no comando: *U${this.apontador + 1}#`)
						this.tarefa = STATE.CAPSULE.INCREMENT;
					break
				}
			break
			case STATE.WAIT.TIMER.CAMERA:
				if(glTempo.state()) {
					glCamera.startTesteCamera();
					emit('isBusy', true);
					this.tarefa = STATE.VERIFY.CAMERA;
					if(!this.flagNReset) this.NRESET = 5;
				}
			break
			case STATE.VERIFY.CAMERA:
				switch (glCamera.statusCamera()) {
					case glCamera.CTOK:
						this.tarefa = STATE.SEND.LED;
					break
					case glCamera.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('CAMERA', 'erro camera não está ligada');
						if(this.NRESET != 0) {
							WLD.log('SERIAL', `reset do hub: ${this.NRESET}`);
							this.NRESET--;
							this.flagNReset = true;
							this.tarefa = STATE.WAIT.RESET;
							WLDSerial.sendSerial(`*R#`);
						} else {
							this.tarefa = STATE.WAIT.USB.CANCEL;
							this.flagNReset = false;
							WLDSerial.sendSerial(`*U0#`);
							this.cancelaEnsaio(false, this.apontador, true);
							Alerta('camera', translate(153).replace('1',this.apontador + 1), translate(26), null, translate(63));
						}
						emit('isBusy', false);
						glCamera.close();
					break
				}
			break
			case STATE.SEND.LED:
				WLDSerial.sendSerial(`*L${this.apontador + 1}#`)
				this.tarefa = STATE.WAIT.LED.ON;
			break
			case STATE.WAIT.LED.ON:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						glTempo.setTimer(6000);
						this.tarefa = STATE.WAIT.TIMER.PHOTO;
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', `erro não teve resposta no comando: *L${this.apontador + 1}#`);
						this.tarefa = STATE.CAPSULE.INCREMENT;
					break
				}
			break
			case STATE.WAIT.TIMER.PHOTO:
				if (glTempo.state()) {
					this.tarefa = STATE.VERIFY.PHOTO;
					glCamera.takePhoto(this.Capsula[this.apontador]);
				}
			break
			case STATE.VERIFY.PHOTO:
				switch (glCamera.statusFoto()) {
					case glCamera.CTOK:

						emit('isBusy', false);
						glCamera.close();

						WLDSerial.sendSerial(`*U0#`);
						WLD.log('SERIAL', '*U0# OK');
						this.tarefa = STATE.WAIT.USB.OFF;
					break
					case glCamera.CTERRO:
						emit('isBusy', false);
						glCamera.close();
						WLDSerial.sendSerial(`*U0#`)
						WLD.log('SERIAL', '*U0# ERRO')
						WLD.log('CAMERA', 'erro de captura de imagem')
						this.tarefa = STATE.WAIT.USB.OFF
					break
				}
			break
			case STATE.WAIT.USB.OFF:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						WLDSerial.sendSerial(`*L0#`);
						this.tarefa = STATE.WAIT.LED.OFF;
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLDSerial.sendSerial(`*L0#`);
						WLD.log('SERIAL', 'erro não teve resposta no comando: *U0#');
						this.tarefa = STATE.WAIT.LED.OFF;
					break
				}
			break
			case STATE.WAIT.LED.OFF:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.tarefa = STATE.VERIFY.HASPHOTO;
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', 'Erro não teve resposta no comando: *L0#');
						this.tarefa = STATE.CAPSULE.INCREMENT;
					break
				}
			break
			case STATE.VERIFY.HASPHOTO:
				this.Capsula[this.apontador].quantidadeDeFoto--

				let loQuantidadeDeFoto = this.Capsula[this.apontador].quantidadeDeFoto;

				if (loQuantidadeDeFoto == 0) {
					this.Capsula[this.apontador].fim = true;
					this.Capsula[this.apontador].flagCtTempo = false;
				} else {
					WLD.log('MAIN', `cavidade: ${this.apontador + 1} e quantidade: ${this.Capsula[this.apontador].quantidadeDeFoto}`);
				}

				this.tarefa = STATE.CAPSULE.INCREMENT;
			break
			case STATE.WAIT.RESET:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.tarefa = STATE.WAIT.TIMER.CAMERA;
						glTempo.setTimer(5000);
					break
					case WLDSerial.CTERRO: // tentou n vezes e nao conseguiu
						WLD.log('SERIAL', `erro não teve resposta no comando: *R#`);
						this.tarefa = STATE.SEND.STATUS;
					break
				}
			break
			case STATE.WAIT.USB.CANCEL:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.tarefa = STATE.SEND.STATUS;
						WLD.log('SERIAL', '*U0# OK');
					break
					case WLDSerial.CTERRO:
						WLD.log('SERIAL', `erro não teve resposta no comando: *U0#`);
						this.tarefa = STATE.SEND.STATUS;
					break
				}
			break
			case STATE.VERIFY.TEMPERATURE:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						
						glTempo.setTimer(1000);
						emit('temperature', { atual: WLDSerial.retorno });

						let x = WLDSerial.retorno;
						let y = this.temperatura?.desejada;

						if (x >= (y - 30) && x <= (y + 30)) {
							this.tarefa = STATE.SEND.STATUS;
						} else {
							this.tarefa = STATE.SEND.STATUS; // modificar apos decidir oq fazer apos sair fora da faixa
							WLD.log('SERIAL', 'Temperatura fora da faixa');
						}
					break
					case WLDSerial.CTERRO: //tentou n vezes e nao conseguiu
						WLD.log('SERIAL', `erro não teve resposta no comando: *T${this.temperatura?.desejada}#`);
						glTempo.setTimer(3000);
						this.tarefa = STATE.SEND.STATUS;
					break
				}
				// verifica se é para atualizar a data
				if(this.tarefa == STATE.SEND.STATUS) {
					if(this.flagDate) {
						this.tarefa = STATE.SEND.DATE.WRITE;
						this.flagDate = false;
					} else {
						if(this.flagBip) {
							this.flagBip = false;

							if(this.flagBipON || this.NBips == 0) {
								WLDSerial.sendSerial(`*B${this.NBips.toString().padStart(4,0)}#`);
								this.tarefa = STATE.WAIT.BUZZER;
							}
						}
					}
				} 
				
			break
			case STATE.SEND.DATE.WRITE:
				WLD.log('SERIAL', `*D${this.newDate}#`);
				WLDSerial.sendSerial(`*D${this.newDate}#`);
				this.tarefa = STATE.WAIT.DATE.WRITE;
			break
			case STATE.WAIT.DATE.WRITE:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.tarefa = STATE.SEND.STATUS;
					break
					case WLDSerial.CTERRO:
						const time = `${WLD.getDate('yyyymmdd')}${WLD.getTime('hhmmss')}`;
						WLD.log('SERIAL', `erro não teve resposta no comando: *D${time}#`)
						this.tarefa = STATE.SEND.STATUS;
					break
				} 
			break
			case STATE.WAIT.BUZZER:
				switch (WLDSerial.leStatus()) {
					case WLDSerial.CTOK:
						this.tarefa = STATE.SEND.STATUS;
					break
					case WLDSerial.CTERRO:
						WLD.log('SERIAL', `erro não teve resposta no comando: *Bxxxx#`);
						this.tarefa = STATE.SEND.STATUS;
					break
				}
			break
			default:
			break
		}

		if(glTempoPrint.state()) {
			this.checkPrint();
			glTempoPrint.setTimer(2000);
		}

		if(this.monthHasChanged()) {

			const isConnected = await WLDWifi.isConnected();

        	if(!isConnected || isConnected == 'error' || isConnected == 'disabled') { 
				WLDNotifications.add({ id: 41, date: `- ${WLD.getTime('hhhmmm')}`, icon: 41 });
			} else {
				const canUpdate = await WLDUpdate.checkUpdate(false);
				if (canUpdate) WLDNotifications.add({ id: 168, date: `- ${WLD.getTime('hhhmmm')}`, icon: 168 });
			}
		}

		BackupController.loop();

		setTimeout(() => this.loop(), 100);
	}

	// #endregion MAIN LOOP

	begin() {
		WLDSerial.begin();

		setTimeout(WLDWifi.loop, 5000);
		setTimeout(() => this.updateDate(), 1000);
		setTimeout(() => this.contadorCapsula(), 5000);

		setTimeout(() => this.loop(), 100);
	}

	close() {
		this.loops.map(loop => clearInterval(loop));
		this.loops = [];
	}
}

module.exports = new WLDMain();