const fs   = require("fs");
const path = require('path');
const pdf  = require("pdf-creator-node");

const WLDTimer  = require('./WLDTimer');
const WLDConfig = require('./WLDConfig');
const WLDBD     = require('./WLDDatabase');
const WLDLanguage = require('./WLDLanguage');
const { WLD } = require("./WLDUtils");

const glConf = WLDConfig;

const options = { format: "A4", orientation: "portrait" }

const TYPE          = 48;
const BATCH         = 58;
const NAME          = 61;
const DURATION      = 75;
const AMPOULE       = 56;
const DATE          = 35;
const HOUR          = 37;
const CYCLE         = 125;
const SERIAL_NUMBER = 115;
const AUTOCLAVE     = 47;
const RESULT        = 49;
const CAVITY        = 74;

const not = {
    "PORTUGUÊS": '/home/pi/wld/pics/common/not_found_portuguese.png',
    "ESPANHOL":  '/home/pi/wld/pics/common/not_found_spanish.png',
    "INGLÊS":    '/home/pi/wld/pics/common/not_found_english.png',
}

const canceled = {
    "PORTUGUÊS": '/home/pi/wld/pics/common/canceled_portuguese.png',
    "ESPANHOL":  '/home/pi/wld/pics/common/canceled_spanish.png',
    "INGLÊS":    '/home/pi/wld/pics/common/canceled_english.png',
}

const translate = (id) => WLDLanguage.translate_printer(id);

class WLDPdf {
    constructor () {
        this.id;
        this.image;
        this.document;
        
        this.glBD    = WLDBD;
        this.glTimer = new WLDTimer();

        this.html         = fs.readFileSync(path.join(__dirname, '../../assets/html/template.html'), 'utf8');
        this.logo_lummina =  fs.readFileSync(path.join(__dirname, '../../assets/html/logo_lummina.png' )).toString('base64'),
        
        this.mode = {
            'maxximed': fs.readFileSync(path.join(__dirname, '../../assets/html/logo_maxximed.png')).toString('base64'),
            'tecil': fs.readFileSync(path.join(__dirname, '../../assets/html/logo_tecil.png' )).toString('base64'),
            'medcontrol': fs.readFileSync(path.join(__dirname, '../../assets/html/logo_med.png' )).toString('base64'),
        } 

        this.sites = {
            'maxximed': 'www.maxximed.com.br',
            'tecil': 'www.baumer.com.br',
            'medcontrol': 'www.medcontrolbauru.com.br',
        } 

        this.type = glConf.read('type', 'mode', 'maxximed');
    }

    

    changeParamns(args, file) {

        try {
            (args['result'] == 3) 
                ? this.image = fs.readFileSync(canceled[WLDLanguage.getCurrentPrinter()]).toString('base64')
                : this.image = fs.readFileSync(`/home/pi/wld/pics/print/${args['id_essay']}.png`).toString('base64');
        } catch (e) {
            this.image = fs.readFileSync(not[WLDLanguage.getCurrentPrinter()]).toString('base64');
        }

        this.type  = glConf.read('type', 'mode', 'maxximed');

        let image_logo = (this.mode[this.type] == undefined) ? this.mode['maxximed'] : this.mode[this.type]
        let site_name = (this.sites[this.type] == undefined) ? this.sites['maxximed'] : this.sites[this.type]

        this.document = {
            html: this.html,
            data: {
                logo: image_logo,
                image: this.image,
                logo_lummina: this.logo_lummina,

                batch: args['batch'],
                user:  args['name'],
                cavity: args['cavity'].toString().padStart(2, 0),

                type: translate(args['type']),
                ampoule: translate(args['ampoule']),

                data: args['date'].substring(0, 10),
                hour: args['date'].substring(13, 21),

                cycle: args['id_essay'].toString().padStart(6, 0),

                nserie: glConf.read('info', 'serial_number', '0000'),

                time: (args['duration'] == 0)
                    ? '00:00:00'
                    : args['duration'],

                autoclave: (args['model'] != 'CONTROLE')
                    ? args['model']
                    : '',
                
                result: (args['result'] == 1) 
                    ? translate(54) 
                    : (args['result'] == 2) 
                        ? translate(55) 
                        : translate(52),

                type_label: translate(TYPE),
                user_label: translate(NAME),
                data_label: translate(DATE),
                hour_label: translate(HOUR),
                cycle_label: translate(CYCLE),
                batch_label: translate(BATCH),
                time_label: translate(DURATION),
                result_label: translate(RESULT),
                ampoule_label: translate(AMPOULE),
                autoclave_label: translate(AUTOCLAVE),
                nserie_label: translate(SERIAL_NUMBER),
                cavity_label: translate(CAVITY),

                site: site_name,

            },
            path: `/media/pi/${file}/lummina pro 8 PDF's/${args['id_essay']}.pdf`,
            type: "",
        }
    }

    async createPDF() {
        try {
            pdf.create(this.document, options);
            return true;
        } catch (error) {
            return false;
        }
    }

    async makePdf(id, save) {
        const ensaio = await this.glBD.getData('default', id);
        if(!ensaio) return false;
        
        this.changeParamns(ensaio, save);
        return await this.createPDF();
	}
}

module.exports = WLDPdf;