const WLDTimer = require('./WLDTimer');
const WLDConfig = require('./WLDConfig');
const SerialPort = require('serialport');
const ReadlineParser = require('@serialport/parser-readline');

const glConf = WLDConfig;

var status;
var comando;
var glComando;
var tarefa = 0;
var vezesEnvio = 0;
const CTTEMPOTIMEROUT = 5000;
var estadoAnterior = '00000000';

class WLDSerial {
    constructor() {

        this.retorno;
        this.statusCapsulas;
        this.temperaturaAtual;
        this.glTempo = new WLDTimer();
        this.WLDLog = (type, value) => console.log(`[${type}] -> ${value}`);

        // constantes
        this.CTOK = 1;
        this.CTERRO = -1;
        this.CTTIMEROUT = -2;
        this.CTVEZESENVIO = 3;
        this.CTERRORECEBE = 4;
        this.CTPROCESSANDO = 0;

        this.loop;
        this.port = new SerialPort('/dev/ttyS0', { baudRate: 9600 });
        // this.port = new SerialPort('/dev/tty.usbserial-1140', { baudRate: 9600 });
        this.parser = this.port.pipe(new ReadlineParser({ delimiter: '#' }));

        this.parser.on('data', data => {
            if(comando == data[1]) {
                switch (data[1]) {
                    case 'S':
                        let loAux = true;

                        for(let loCt = 0; loCt < 8; loCt++) {
                            if(data[loCt+2] !== '0' && data[loCt+2] !== '1') {
                                loAux = false;
                                status = this.CTERRORECEBE;
                                break
                            }
                        }

                        if (data.length !== 10) {
                            loAux = false;
                            status = this.CTERRORECEBE;
                        }

                        if (loAux) {
                            this.retorno = data.substring(2);
                            status = this.CTOK;
                        }

                        if (this.retorno != estadoAnterior) {
                            estadoAnterior = this.retorno;
                            this.WLDLog('SERIAL',`sensor capsula: ${estadoAnterior}`);
                        }

                        tarefa = 0;
                        break
                    case 'T':
                        data = data.substring(2);

                        let isNum = /^\d+$/.test(data);

                        if(!isNum) {
                            status = this.CTERRO;
                            break
                        }

                        let offset = glConf.read('calibrate', 'temperature_offset', 0);

                        status = this.CTOK;
                        this.retorno = parseInt(data) + offset * 10;

                        tarefa = 0;
                        break
                    case 'L':
                        if (data != '*L') {
                            status = this.CTERRO;
                            break
                        }

                        tarefa = 0;
                        status = this.CTOK;
                        // this.retornoSerial = true;
                        break
                    case 'U':
                        if (data != '*U') {
                            status = this.CTERRO;
                            break
                        }

                        tarefa = 0;
                        status = this.CTOK;
                        // this.retornoSerial = true;
                    break
                    case 'B':
                        tarefa = 0;
                        status = this.CTOK;
                    break
                    case 'D':
                        const loRet = data.substring(2);
                        const isNumber = /^\d+$/.test(loRet);

                        if(isNumber && loRet.length == 14) {
                            this.retorno = loRet;
                            status = this.CTOK;
                            tarefa = 0;
                        } else {
                            status = this.CTERRO;
                        }
                    break
                    case 'R':
                        if (data != '*R') {
                            status = this.CTERRO;
                            break
                        }

                        tarefa = 0;
                        status = this.CTOK;
                    break
                }
            }
        })
    }

    sendSerial(paComando) {
        vezesEnvio = this.CTVEZESENVIO;
        status = this.CTPROCESSANDO;
        this.send(paComando);
    }

    leEstado = () => estadoAnterior;

    leStatus = () => status;

    begin() {
        this.loop = setInterval(() => {
            switch (tarefa) {
                case 0: // Aguardando
                break
                default:
                    if (this.glTempo.state()) {
                        this.WLDLog('SERIAL',`re-enviando comando, vezes restantes: ${vezesEnvio}.`)
                        if (vezesEnvio == 0) {
                            status = this.CTERRO
                            tarefa = 0
                            break
                        }

                        vezesEnvio--
                        this.WLDLog('SERIAL',`re-enviando comando: ${glComando}.`)
                        this.send(glComando)
                    }
                break
            }
        }, 100)
    }

    checkState = async () => (estadoAnterior !== '00000000') ? false : true;

    close() {
        clearInterval(this.loop)
        this.loop = null
    }

    send(paComando) {
        glComando = paComando;
        comando = paComando[1];
        this.glTempo.setTimer(CTTEMPOTIMEROUT)
        this.port.write(paComando, err => (err) && this.WLDLog('SERIAL',`erro no envio do comando: ${paComando}, erro: ${err}`))
        tarefa = 1
    }

}

module.exports = new WLDSerial();