const fs = require('fs');
const Client = require('ftp');
const progress = require('progress-stream');

const { WLD } = require('./WLDUtils');
const { Alerta } = require('./WLDAlert');
const { execSync } = require('child_process');
const { translate } = require('./WLDLanguage');
const WLDConfig = require('./WLDConfig');
const WLDIO = require('./WLDIO');

class WLDUpdate {
    constructor() {
        this.client;
        this.folder = WLDConfig.read('type','folder', 'production');
        this.fileChanged = false;
        this.version = { new: '',  current: '' }
        this.hash = { downloaded: '', file: '' }
        this.download_folder = '/home/pi/wld/download/';
    }

    async checkUpdate(notify) {

        this.folder = WLDConfig.read('type','folder', 'production');
        this.connect();

        Alerta('camera', translate(165), translate(164), null, null, notify);
        const isConnected = await this.isConnected();
        
        if(!isConnected) return this.disconnect('sem conexão com o FTP');
        
        Alerta('camera', translate(166), translate(164), null, null, notify);
        const listing = await this.list();
        
        if(listing.length == 0) return this.disconnect('nenhum arquivo no FTP');
        
        let loIndex;
        
        listing.map((file, index) => (file.name == 'version.json') && (loIndex = index));
        
        Alerta('camera', translate(167), translate(164), null, null, notify);
        const doneDownloadingVersion = await this.downloadFile(listing[loIndex].name, listing[loIndex].size, false);
        
        if(!doneDownloadingVersion) return this.disconnect('erro ao fazer download do arquivo version.json');
        
        Alerta('camera', 'comparando as versões.', translate(164), null, null, notify);
        const isNewerVersion = await this.compareVersions();

        if(!isNewerVersion) return this.disconnect('mesma versão, update não necessario', false);

        this.disconnect();

        return true;
    }

    async makeDownload() {
        this.connect();

        const isConnected = await this.isConnected();

        if(!isConnected) return this.disconnect('sem conexão com o FTP');

        const listing = await this.list();

        if(listing.length == 0) return this.disconnect('nenhum arquivo no FTP');

        let loIndex;

        listing.map((file, index) => (file.name == 'update.zip') && (loIndex = index));

        // Alerta('camera', translate(169), translate(164), null, null);
        
        const doneDownloadingUpdate = await this.downloadFile(listing[loIndex].name, listing[loIndex].size);

        if(!doneDownloadingUpdate) return this.disconnect('erro ao fazer download do arquivo update.zip');

        Alerta('camera', 'verificando integridade do arquivo.', translate(164), null, null);

        const isSameHash = await this.compareHashes();

        if(!isSameHash) return this.disconnect('erro arquivo modificado, hashes não bateram');

        // const writing = await this.writeJSON();

        // if(!writing) return this.disconnect('erro ao gravar o arquivo version.json');

        const renamingUpdate = await this.renameFile('downloaded_update.zip');

        if(!renamingUpdate) return this.disconnect('erro ao renomear arquivo');

        const renamingVersion = await this.renameFile('downloaded_version.json');

        if(!renamingVersion) return this.disconnect('erro ao renomear arquivo');
        
        try { execSync('rm /home/pi/wld/update.*'); }
        catch (error) { }
        
        Alerta('camera', translate(175), translate(164), null, null);
        this.disconnect();

    }

    // async makeDownload() {
    //     await WLDIO.copyFile('/home/pi/wld/upload/version.json', '/home/pi/wld/download/version.json');
    //     await WLDIO.copyFile('/home/pi/wld/upload/update.zip', '/home/pi/wld/download/update.zip');
    //     try { execSync('rm /home/pi/wld/update.*'); }
    //     catch (error) { }

    //     Alerta('camera', 'update baixado com sucesso, por favor retire o equipamento da tomada e aguarde alguns instantes para religa-lo', translate(164), null, null);
    // }

    // async begin() {
    //     this.connect();

    //     const isConnected = await this.isConnected();

    //     if(!isConnected) return this.disconnect('sem conexão com o FTP');

    //     const listing = await this.list();

    //     if(listing.length == 0) return this.disconnect('nenhum arquivo no FTP');

    //     let loIndex;

    //     listing.map((file, index) => (file.name == 'version.json') && (loIndex = index));

    //     const doneDownloadingVersion = await this.downloadFile(listing[loIndex].name, listing[loIndex].size);

    //     if(!doneDownloadingVersion) return this.disconnect('erro ao fazer download do arquivo version.json');

    //     const isNewerVersion = await this.compareVersions();

    //     if(!isNewerVersion) return this.disconnect('mesma versão, update não necessario');

    //     listing.map((file, index) => (file.name == 'update.zip') && (loIndex = index));

    //     const doneDownloadingUpdate = await this.downloadFile(listing[loIndex].name, listing[loIndex].size);

    //     if(!doneDownloadingUpdate) return this.disconnect('erro ao fazer download do arquivo update.zip');

    //     const isSameHash = await this.compareHashes();

    //     if(!isSameHash) return this.disconnect('erro arquivo modificado, hashes não bateram');

    //     const writing = await this.writeJSON();

    //     if(!writing) return this.disconnect('erro ao gravar o arquivo version.json');

    //     this.disconnect();
    // }

    connect() {
        this.client = new Client();
        try {
            this.client.connect({
                port: 21,
                host: 'www.plassa.com.br',
                user: 'plassa@plassa.com.br',
                password: 'Plassa2020Plassa'
            });
            return true;
        } catch (error) {
            return false;            
        }
        
    }

    disconnect = (message = '', notify = true) => {
        if(message != '') Alerta('camera', message, translate(164), null, translate(11), notify);
        // if(message != '') WLD.log('camera', message);
        // WLD.log(`FTP`, 'desconectando do servidor.');
        this.client.end();
    }

    async upload(file, folder) {
        
        WLD.log(`FTP`, `lendo arquivo: ${file}`);
        const file_update = fs.createReadStream(`/home/pi/wld/upload/${file}`);
        
        return new Promise((resolve) => {
            WLD.log(`FTP`, `fazendo upload do arquivo: ${file}`);
            this.client.put(file_update, `${folder}/${file}`, false, (err) => {
                if (err) {
                    WLD.log(`FTP`, `erro: ${err}`);
                    resolve(false);
                }
                // this.disconnect();
                resolve(true);
            });
        });
    }

    async makeUpload(folder) {

        WLD.log(`FTP`, 'conectando');
        this.connect();
        
        const isConnected = await this.isConnected();
        
        if(!isConnected) return this.disconnect('sem conexão com o FTP');
        WLD.log(`FTP`, 'esta conectado');

        const version_uploaded = await this.upload('version.json', folder);

        if(!version_uploaded) return WLD.log(`FTP`, 'erro ao fazer upload do arquivo version.json');
        WLD.log(`FTP`, `upload concluido`);
        
        const update_uploaded = await this.upload('update.zip', folder);
        
        if(!update_uploaded) return WLD.log(`FTP`, 'erro ao fazer upload do arquivo update.zip');
        WLD.log(`FTP`, `upload concluido`);

        this.disconnect();

        return true;
    }

    isConnected() {
        return new Promise((resolve) => {
            this.client.on('ready', err => {
                if (err)  resolve(false);
                // WLD.log('FTP', 'conectou ao servidor.');
                resolve(true)
            });
        });
    }

    list() {
        return new Promise((resolve) => {
            this.client.list(this.folder, (err, listing) => {
                if (err) resolve(false);
                // WLD.log('FTP','listou arquivos: ');
                // listing.map(file => WLD.log('FTP', `file: ${file.name}`));
                resolve(listing);
            });
        });
    }

    async compareVersions() {
        this.version.new     = WLDIO.readVersion(this.download_folder + `downloaded_version.json`)?.version;
        this.version.current = WLDIO.readVersion('/home/pi/wld/lummina/src/version.json')?.version;
        
        if (this.version.current == this.version.new) return false;

        return true;
    }

    downloadFile(file, size, notify = true) {
        return new Promise((resolve) => {
            this.client.get(`${this.folder}/${file}`, (err, stream) => {
                if (err) resolve(false);

                const str = progress({ length: size, time: 100 });

                str.on('progress', ({ percentage }) => Alerta('camera', `${translate(169)}, ${Math.round(percentage)}%`, translate(164), null, null, notify));

                stream.once('close', () => resolve(true));

                try { 
                    stream
                    .pipe(str)
                    .pipe(fs.createWriteStream(`${this.download_folder}downloaded_${file}`))
                    .on('error', () => resolve(false)); 
                } catch (err) { resolve(false) }
            })
        });
    }

    async compareHashes() {

        this.hash.downloaded = JSON.parse(fs.readFileSync(this.download_folder + `downloaded_version.json`)).hash;

        this.hash.file = execSync('md5sum /home/pi/wld/download/downloaded_update.zip').toString().split(' ')[0];

        if(this.hash.file == '') return false;
            
        if (this.hash.file !== this.hash.downloaded) this.fileChanged = true;

        if (this.fileChanged) return false;

        return true;
    }

    async renameFile(file) {
        try {
            execSync(`mv /home/pi/wld/download/${file} /home/pi/wld/download/${file.split('_')[1]}`);
            return true;
        } catch (err) {
            return false;
        }
    }    
}

module.exports = new WLDUpdate();
