const fs = require('fs');
const path = require('path');

const WLDConfig = require('./WLDConfig');

const { emit } = require("./WLDWindow");
const { getCurrentPrinter, translate_printer, translate } = require("./WLDLanguage");

class WLD {

    log = (type, value) => console.log(`[${type}] -> ${value}`);

    Debug = (type, value) => console.log(`[${type}] -> ${JSON.stringify(value)}`);
	
	delay = (ms) => new Promise(resolve => setTimeout(resolve, ms));

	getTime(pattern = '') {
		const date = new Date();

		let customPattern = pattern;

		const hour = date.getHours().toString().padStart(2, 0);
		const minute = date.getMinutes().toString().padStart(2, 0);
		const seconds = date.getSeconds().toString().padStart(2, 0);

		customPattern = customPattern.toString().replace('hh', hour);
		customPattern = customPattern.toString().replace('mm', minute);
		customPattern = customPattern.toString().replace('ss', seconds);

		if(customPattern == pattern) return { hour, minute, seconds };
	
		return customPattern;
	}

	getDate(pattern = '') {
		const date = new Date();
	
		let customPattern = pattern;
	
		const day = date.getDate().toString().padStart(2, 0);
		const year = date.getFullYear().toString().padStart(2, 0);
		const month = (date.getMonth() + 1).toString().padStart(2, 0);
	
		customPattern = customPattern.toString().replace('dd', day);
		customPattern = customPattern.toString().replace('mm', month);
		customPattern = customPattern.toString().replace('yyyy', year);
		customPattern = customPattern.toString().replace('yy', year.substring(2,4));
	
		if(customPattern == pattern) return { day, month, year };
	
		return customPattern;
	}

	verifyNull = (value, placeholder) => (value != null) ? value : placeholder;

	factory(type, args) {

		const { result, date: datetime, duration } = args;

		const _data = {};

		const _results = {
			0: '',
			1: 54,
			2: 55,
			3: 52
		}

		switch(type) {
			case 'print':

				const loData = datetime.substring(0,10);
				const loHora = datetime.substring(13,18);
				const loTime = `${this.verifyNull(duration[0], 0)}${this.verifyNull(duration[1], 0)}h${this.verifyNull(duration[3], 0)}${this.verifyNull(duration[4], 0)}m`;


				_data.cavity = args.cavity + 1;
				_data.type = translate_printer(args.type)

				_data.date = loData,
				_data.hour = loHora,
				_data.time = loTime,
				_data.cycle = args.cycle;
				_data.user = args.name,
				_data.batch = args.batch,
				_data.ampoule = translate_printer(args.ampoule),
				_data.number = WLDConfig.read('info','serial_number', '0000'),
				_data.incub = args.id_essay,
				_data.equip = (args.model == 'CONTROLE') ? '' : args.model,
				_data.image = args.id_essay,

                _data.result = translate_printer(_results[result]);

			break
		}

		return _data;
	}

	convSegHHMMSS(duration) {
		const hours_in_day = 24;
		const second_in_ms = 1000;
        const minutes_in_hour = 60;
        const minute_in_seconds = 60;

		// const milliseconds = String(Math.floor((duration % 1000) / 100)).padStart(2,0)
		const seconds = String(Math.floor((duration /  second_in_ms) % 60)).padStart(2,0);
		const minutes = String(Math.floor((duration / (second_in_ms  * minute_in_seconds)) % 60)).padStart(2,0);
		const hours   = String(Math.floor((duration / (second_in_ms  * minute_in_seconds   * minutes_in_hour)) % hours_in_day)).padStart(2,0);

		return this.mask(`${hours}${minutes}${seconds}`,'##:##:##')
	}

	mask(value, pattern) {
		let i = 0;
		const v = value.toString();
		return pattern.replace(/#/g, () => v[i++] || '');
	}

	readImageOnDisk({ img, type, status }) {

		const pics_path = path.join(__dirname + '../../../../../pics'); 

		let image_path = `${pics_path}/ensaios/${img}.jpg`;

		const not = {
			"PORTUGUÊS": `${pics_path}/common/not_found_portuguese.png`,
			"ESPANHOL":  `${pics_path}/common/not_found_spanish.png`,
			"INGLÊS":    `${pics_path}/common/not_found_english.png`,
		}

		const canceled = {
			"PORTUGUÊS": `${pics_path}/common/canceled_portuguese.png`,
			"ESPANHOL":  `${pics_path}/common/canceled_spanish.png`,
			"INGLÊS":    `${pics_path}/common/canceled_english.png`,
		}

		if (typeof img == 'string') image_path = `${pics_path}/${img}.jpg`;

		if (status == 52 || status == translate(52)) image_path = canceled[getCurrentPrinter()];

		try {
			var _img = fs.readFileSync(image_path).toString('base64');
		} catch ({ message }) {

			const error_image = fs.readFileSync(not[getCurrentPrinter()]).toString('base64');

			this.log('MAIN',`erro ao ler imagem, erro: ${type}`);
			return emit(type, `data:image/jpg;base64,${error_image}`);
		}

		emit(type, `data:image/jpg;base64,${_img}`);

	}

	async generateQRCode(text) {
		try {
			return await QRCode.toDataURL(text);
		} catch (err) {
			console.error(err);
		}
	}


}

module.exports = { WLD: new WLD() };