const { execSync } = require('child_process');
const { emit } = require('./WLDWindow');
const os = require('os');

const STATE = {
	error: 'error',
	connected: true,
	disconnected: false,
	disabled: 'disabled',
}

const has = (item) => item.trim().split(' ')[0] == '*';

const find = (element) => {
	if (has(element)) {
		if (this.state != STATE.disabled) {
			this.state = STATE.connected;
			return element;
		}
	}
}

class WLDWifi {

    constructor() {
        this.state = STATE.disconnected;
    }

	getState = () => this.state;
	setState = (value) => (value) ? this.state = STATE.disconnected : this.state = STATE.disabled;

	listNetworks = () => {
		try {
			return execSync('nmcli device wifi', { stdio : 'pipe' }).toString().split(/\r?\n|\r|\n/g);
		} catch (error) {
			return false;	
		}
	}

	listCon = () => {
		const cons = execSync('nmcli con show', { stdio : 'pipe' }).toString().split(/\r?\n|\r|\n/g);

		let filter = [];
		let lo_start, lo_end;

		cons.forEach((element, index) => {
			if(index === 0) { 
				lo_start = element.search('NAME');
				lo_end   = element.search('UUID');
				return;
			}

			filter.push(element.substring(lo_start, lo_end).trim());
		});

		filter.pop();
		filter.shift();

		return filter;
	}

    scan = () => {
		const networks = this.listNetworks();

		let filter = [];
		let lo_start, lo_end;

		networks.forEach((element, index) => {
			if(index === 0) { 
				lo_start = element.search('SSID');
				lo_end   = element.search('MODE');
				return;
			}

			filter.push(element.substring(lo_start, lo_end).trim());
		});

		filter.pop()

		return filter;
	}

	isWifiConnected = () => {
		const networks = this.listNetworks();
		networks.find(find);
		return this.state;
	}

	getConnectedNetwork = () => {
		const networks = this.listNetworks();
		let network;
		
		if (os.platform() !== 'darwin') network = networks.find(find);
		
		return network?.toString().split(' ').filter(item => item != '')[1];
	}
    
    delete = (rede) => {
		try {
			execSync(`nmcli con delete "${rede}"`, { stdio : 'pipe' });
		} catch (error) {
			// WLD.log(`WIFI`, error);
		}
	}
    
    connect = async (rede, pwd) => {
		try {
			return execSync(`nmcli device wifi con "${rede}" password "${pwd}"`, { stdio : 'pipe' }).toString();
		} catch (error) {
			this.state = STATE.disconnected;
			return false;
		}
	}

    isConnected = async () => {
		try {
			return (this.state == STATE.disabled) ? 'disabled' : execSync('ping -c 1 www.google.com').toString().includes('1 received');
		} catch (error) {
			return 'error';
		}
	}

	loop = async () => {
		emit('isBusyBlue', true);

		try {
			const result = await this.isConnected();
			(this.state != STATE.disabled) && (this.state = (result == true) ? STATE.connected : STATE.disconnected);
		} catch (error) {
			this.state = STATE.error;
		}

		emit('wifi', this.state);
		emit('wifiConnected', this.getConnectedNetwork());
		emit('isBusyBlue', false);
		setTimeout(this.loop, 300000);
	}

}

module.exports = new WLDWifi();