const path = require('path');
const sqlite3 = require('sqlite3').verbose();
const dbPath = path.join(__dirname, '../../../database/database.db');

const _command = {
    edit: (table, { _old, _new, _id, _type }) => `UPDATE ${table} SET ${_old} = '${_new}' WHERE ${_type} = ${_id}`,
    print: `SELECT idTeste FROM 'print' ORDER BY idPrint ASC LIMIT 1`,
    default: `SELECT En.NIncubacao, En.resultado, En.tipo, En.lote, En.idUsuario, En.idEquipamento, En.cavidade, En.duracao, En.data,
    En.ciclo, En.Ampola, U.nome, U.tipo as usertipo, Eq.modelo, Eq.codigo FROM ensaios AS En JOIN usuario AS U ON U.idUsuario = En.idUsuario
    JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`
}

const _insert = {
    print:  (id)                               => `INSERT INTO print (idTeste) VALUES ('${id}')`,
    user:   ({ user:  { name, pwd, type }})    => `INSERT INTO usuario (nome, senha, tipo) VALUES ('${name}','${pwd}','${type}')`,
    equip:  ({ equip: { brand, model, code }}) => `INSERT INTO equipamento (marca, modelo, codigo) VALUES ('${brand}','${model}','${code}')`,
    essays: ({ essays })                       => `INSERT INTO ensaios (resultado, tipo, lote, idUsuario, idEquipamento, cavidade, duracao, data, ciclo, ampola) VALUES
    ('${essays.result}','${essays.type}','${essays.batch}','${essays.id_user}','${essays.id_equip}', '${essays.cavity}','${essays.duration}','${essays.date}','${essays.cicle}','${essays.ampoule}')`,
}

const _get = {
    print:    (db)           => db.get(_command['print'], (err, row) => {}),
    user:     (db, table)    => db.all(`SELECT * FROM ${table}`, (err, row) => { }),
    equip:    (db, table)    => db.all(`SELECT * FROM ${table}`, (err, row) => { }),
    all_pdfs: (db)           => db.all(`SELECT En.NIncubacao AS "N INCUB" FROM ensaios AS En JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`, (err, row) => { }),
    verify:   (db, type, id) => db.get(`SELECT count(${type}) from ensaios where ${type} = ${id}`, (err, count) => count[`count(${type})`]),
    default:  (db, id)       => db.get(_command['default'] + ` WHERE En.NIncubacao = ${id}`, (err, row) => {}),
    essays:   (db)           => db.get(`SELECT NIncubacao FROM 'ensaios' ORDER BY NIncubacao DESC LIMIT 1`, (err, row) => {}),
    
    all_essays: (db) => db.all(`SELECT En.NIncubacao, En.resultado, En.tipo, En.cavidade, En.idUsuario, En.idEquipamento, En.duracao, En.data, Eq.modelo,
            Eq.codigo FROM ensaios AS En JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`, (err, row) => { }),
    csv_essays: (db) => db.all(`SELECT En.NIncubacao AS "N INCUB", En.data AS "DATA / HORA", CASE En.duracao WHEN '0' THEN '00:00:00' ELSE En.duracao END AS "DURAÇÃO", 
        Eq.modelo AS "AUTOCLAVE", En.tipo AS "TIPO", 
        CASE En.resultado 
            WHEN '3' THEN 'CANCELADO'
            WHEN '2' THEN 'NEGATIVO'
            WHEN '1' THEN 'POSITIVO'
            WHEN '0' THEN 'EM ANDAMENTO'
            ELSE 'ERRO' END AS "RESULTADO" 
            FROM ensaios AS En JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`,
            (err, row) => { }),
}

const create = {
    print:  `CREATE TABLE print (idPrint INTEGER PRIMARY KEY, idTeste INTEGER NOT NULL)`,
    user:   `CREATE TABLE usuario (idUsuario INTEGER PRIMARY KEY, nome TEXT NOT NULL, senha TEXT NOT NULL, tipo INTEGER NOT NULL)`,
    equip:  `CREATE TABLE equipamento (idEquipamento INTEGER PRIMARY KEY, marca TEXT NOT NULL, modelo TEXT NOT NULL, codigo TEXT NOT NULL)`,
    essays: `CREATE TABLE ensaios (NIncubacao INTEGER PRIMARY KEY, resultado INTEGER NOT NULL, tipo INTERGER NOT NULL, lote TEXT NOT NULL, ciclo TEXT NOT NULL, ampola TEXT NOT NULL, idUsuario INT NOT NULL, idEquipamento INT NOT NULL, cavidade INTEGER NOT NULL, duracao REAL NOT NULL, data DATE NOT NULL, FOREIGN KEY (idUsuario) REFERENCES usuario(id), FOREIGN KEY (idEquipamento) REFERENCES equipamento(id))`,
}

const Err      = (err)                 => (err) && console.error(err);
const cbOR     = (err, rej, res)       => (err) ? rej({ state: false, error: err }) : res(true);
const cbReturn = (err, data, rej, res) => (err) ? rej({ state: false, error: err }) : res(data);
const cbEdita  = (err, rej)            => (err) && rej(console.error(`[BANCO DE DADOS] -> erro ao editar dado: ${err.message}`));

class WLDBD {

    CriaBancoDados() {
        const db = new sqlite3.Database(dbPath, Err);

        db.serialize(() => 
            db.run(create['user'])
              .run(create['equip'])
              .run(create['print'])
              .run(create['essays']));

        db.close(Err);
    }

    InsereNovoDado(paTabela, paData) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        return new Promise((resolve, reject) => {
            switch(paTabela) {
                case 'usuario':
                    const { nome: loUserName, senha: loUserPwd, tipo: loUserType } = paData
                    db.run(`INSERT INTO ${paTabela}
                        (nome, senha, tipo) VALUES
                        ('${loUserName}','${loUserPwd}','${loUserType}')`)
                break
                case 'equipamento':
                    const { marca: loEquipMarca, modelo: loEquipModelo, codigo: loEquipCodigo } = paData
                    db.run(`INSERT INTO ${paTabela}
                        (marca, modelo, codigo) VALUES
                        ('${loEquipMarca}','${loEquipModelo}','${loEquipCodigo}')`)
                break
                case 'ensaios':
                    const { ampola: paAmpola } = paData
                    const { duracao: loDuration, data: loDate, ciclo: paCiclo, } = paData
                    const { resultado: loResultado, tipo: loTipo, lote: loLote, } = paData
                    const { idUser: loIdUser, idEquip: loIdEquip, cavidade: loCavidade} = paData
                    db.run(`INSERT INTO ${paTabela}
                        (resultado, tipo, lote, idUsuario, idEquipamento,
                            cavidade, duracao, data, ciclo, ampola) VALUES
                        ('${loResultado}','${loTipo}','${loLote}','${loIdUser}','${loIdEquip}',
                        '${loCavidade}','${loDuration}','${loDate}','${paCiclo}','${paAmpola}')`)
                break
                case 'print':
                    db.run(`INSERT INTO ${paTabela} (idTeste) VALUES ('${paData}')`)
                break
            }

            db.close((err) => cbOR(err, reject, resolve));
        })
    }

    EditaDado(paTabela, paData) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err)
        return new Promise((resolve, reject) => {

            const { dadoAntigo: oldData, dadoNovo: newData, id } = paData;

            const _data = {
                _old: oldData,
                _new: newData,
                _id: id,
                type: (paTabela == 'ensaios') ? 'NIncubacao' : (paTabela == 'equipamento') ? 'idEquipamento' : 'idUsuario'
            }

            db.run(_command['edit'](paTabela, _data), (err) => cbEdita(err, reject))
            db.close((err) => cbOR(err, reject, resolve))
        })
    }

    AcertaBD() { //acerta ensaios
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err)
        return new Promise((resolve, reject) => {
            db.run('UPDATE ensaios SET resultado = 3 WHERE duracao = 0 AND resultado = 0', (err) => cbEdita(err, reject))
            db.close((err) => cbOR(err, reject, resolve))
        })
    }

    DeletaFila(paId) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err)
        return new Promise((resolve, reject) => {
            db.run(`DELETE FROM print WHERE idTeste = ${paId}`)
            db.close((err) => cbOR(err, reject, resolve))
        })
    }

    DeletaDado(paTabela, paId, paNID = 'NIncubacao') {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err)
        return new Promise((resolve, reject) => {
            db.run(`DELETE FROM ${paTabela} WHERE ${paNID} = ${paId}`)
            db.close((err) => cbOR(err, reject, resolve))
        })
    }

    PegaDado(paTabela, paNum, paType) {

        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READONLY, Err)
        return new Promise(async (resolve, reject) => {
            switch(paTabela) {
                case 'ensaios':
                    db.get(`SELECT NIncubacao FROM 'ensaios' ORDER BY NIncubacao DESC LIMIT 1`,
                        (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'allEnsaios':
                    db.all(`SELECT En.NIncubacao,
                        En.resultado AS "result", En.tipo AS "type", En.cavidade,
                        En.idUsuario, En.idEquipamento,
                        En.duracao AS "duration", En.data AS "date", Eq.modelo,
                        Eq.codigo FROM ensaios AS En
                        JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`,
                        (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'csvEnsaios':
                    db.all(`SELECT En.NIncubacao AS "N INCUB",
                    En.data AS "DATA / HORA",
                    CASE En.duracao WHEN '0' THEN '00:00:00' ELSE En.duracao END AS "DURAÇÃO", 
                    Eq.modelo AS "AUTOCLAVE", En.tipo AS "TIPO", 
                    CASE En.resultado 
                        WHEN '3' THEN 'CANCELADO'
                        WHEN '2' THEN 'NEGATIVO'
                        WHEN '1' THEN 'POSITIVO'
                        WHEN '0' THEN 'EM ANDAMENTO'
                        ELSE 'ERRO' END AS "RESULTADO" 
                        FROM ensaios AS En JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`,
                        (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'allPDFs':
                    db.all(`SELECT En.NIncubacao AS "N INCUB" FROM ensaios AS En
                        JOIN equipamento AS Eq ON En.idEquipamento = Eq.idEquipamento`,
                        (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'usuario':
                    db.all(`SELECT * FROM ${paTabela}`, (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'equipamento':
                    db.all(`SELECT * FROM ${paTabela}`, (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'print':
                    db.get(_command['print'], (err, row) => cbReturn(err, row, reject, resolve))
                break
                case 'verify':
                    db.get(`SELECT count(${paType}) FROM ensaios WHERE ${paType} = ${paNum}`, (err, count) => {
                        (err) ? reject({ state: false, error: err }) : resolve(count[`count(${paType})`]);
                    });
                break
                default:
                    db.get(_command['default'] + ` WHERE En.NIncubacao = ${paNum}`,
                        (err, row) => cbReturn(err, row, reject, resolve))
                break
            }

            db.close(Err);
        })
    }
}

module.exports = WLDBD;