const fs = require('fs');
const path = require('path');
const sqlite3 = require('sqlite3');

const { WLD } = require('./WLDUtils');

const dbPath = '/home/pi/wld/database/database.db';
// const dbPath = '/Users/ernesto/Downloads/down/database/database.db';

const csv_sqlfile = readQuery('get/csv.sql');
const pdf_sqlfile = readQuery('get/pdf.sql');
const print_sqlfile = readQuery('get/print.sql');
const essay_sqlfile = readQuery('get/essay.sql');
const users_sqlfile = readQuery('get/users.sql');
const qrcode_sqlfile = readQuery('get/qrcode.sql');
const essays_sqlfile = readQuery('get/essays.sql');
const equips_sqlfile = readQuery('get/equips.sql');
const default_sqlfile = readQuery('get/default.sql');

function readQuery(query_path) {
    const filePath = path.join(__dirname, '../DEV/queries/' + query_path);
    return fs.readFileSync(filePath, { encoding: 'utf-8' });
}

const _command = {
    edit: (table, { _old, _new, _id, _type }) => `UPDATE ${table} SET ${_old} = '${_new}' WHERE ${_type} = ${_id}`,
}

const _insert = {
    prints: (id)                     => `INSERT INTO prints (id_essay) VALUES ('${id}')`,
    users:  ({ name, pwd, type })    => `INSERT INTO users  (name, pwd, type) VALUES ('${name}','${pwd}','${type}')`,
    equips: ({ brand, model, code }) => `INSERT INTO equips (brand, model, code) VALUES ('${brand}','${model}','${code}')`,
    essays: (essays)                 => `INSERT INTO essays (result, type, batch, id_user, id_equip, cavity, duration, date, cycle, ampoule) VALUES
    ('${essays.result}','${essays.type}','${essays.batch}','${essays.id_user}','${essays.id_equip}', '${essays.cavity}','${essays.duration}','${essays.date}','${essays.cycle}','${essays.ampoule}')`,
}

const _get = {
    verify: ({ db, res, type, id }) => db.get(`SELECT count(${type}) FROM essays WHERE ${type} = ${id}`, (_, count) => res(count[`count(${type})`])),

    default:    ({ db, res, id }) => db.get(default_sqlfile, [id], (_, row) => res(row, res)),
    qrcode:     ({ db, res, id }) => db.get(qrcode_sqlfile,  [id], (_, row) => verify(row, res)), 
    prints:     ({ db, res })     => db.get(print_sqlfile,  (_, row) => verify(row, res)),
    essays:     ({ db, res })     => db.get(essay_sqlfile,  (_, row) => res(row)),
    
    users:      ({ db, res })     => db.all(users_sqlfile,  (_, row) => res(row)),
    equips:     ({ db, res })     => db.all(equips_sqlfile, (_, row) => res(row)),
    all_pdfs:   ({ db, res })     => db.all(pdf_sqlfile,    (_, row) => res(row)),
    csv_essays: ({ db, res })     => db.all(csv_sqlfile,    (_, row) => res(row)),
    all_essays: ({ db, res })     => db.all(essays_sqlfile, (_, row) => res(row)),
}

const Err = (err) => err && console.error(err);
const verify = (data, res) => (data !== undefined) ? res(data): res(false)

class WLDBD {

    static database_location = '/Users/ernesto/Downloads/down/database/database.db';
    static clean_database_location = '/Users/ernesto/Downloads/down/database/database_clean.db';
    // static database_location = '/home/pi/wld/database/database.db';
    // static clean_database_location = '/home/pi/wld/database/database_clean.db';

    async createDatabase() {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        const users_sqlfile  = readQuery('create/users.sql');
        const equips_sqlfile = readQuery('create/equips.sql');
        const prints_sqlfile = readQuery('create/prints.sql');
        const essays_sqlfile = readQuery('create/essays.sql');

        try {
            db.serialize(() => {
                db.run(users_sqlfile);
                db.run(equips_sqlfile);
                db.run(prints_sqlfile);
                db.run(essays_sqlfile);
            });
            db.close(Err);
            return true;
        } catch (error) {
            Err(error);
            db.close(Err);
            return false;
        }
    }

    async insertNewData(table, data) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        try {
            db.run(_insert[table](data));
            db.close(Err);
            return true;
        } catch (error) {
            Err(error);
            db.close(Err);
            return false;
        }
    }

    async editData(table, data) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        try {
            db.run(_command['edit'](table, data), (err) => {});
            db.close(Err);
            return true;
        } catch (error) {
            Err(error);
            db.close(Err);
            return false;
        }

    }

    async correctDatabase() {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        try {
            db.run('UPDATE essays SET result = 3 WHERE duration = 0 AND result = 0', Err);
            db.close(Err);
            return true;
        } catch (error) {
            Err(error);
            db.close(Err);
            return false;
        }
    }

    async deleteQueue(id) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        try {
            db.run(`DELETE FROM prints WHERE id_essay = ${id}`);
            db.close(Err);
            return true;
        } catch (error) {
            Err(error);
            db.close(Err);
            return false;
        }
    }

    async deleteData(table, id, selector = 'id_essay') {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        try {
            db.run(`DELETE FROM ${table} WHERE ${selector} = ${id}`)
            db.close(Err);
            return true;
        } catch (error) {
            Err(error);
            db.close(Err);
            return false;
        }

    }

    async getData(table, id, type) {
        const db = new sqlite3.Database(dbPath, sqlite3.OPEN_READWRITE, Err);

        return new Promise((resolve) => {

            const _data = { db, res: resolve, type, id, table };

            try {
                _get[table](_data);
                db.close(Err);
            } catch (error) {
                Err(error);
                db.close(Err);
                resolve(false);
            }
        })

    
    }

}

module.exports = WLDBD;