const { WLD } = require("./WLD/WLDUtils");

const WLDMain = require("./WLD/WLDMain");
const WLDAlert = require("./WLD/WLDAlert");
const WLDWindow = require("./WLD/WLDWindow");
const WLDNotifications = require("./WLD/WLDNotifications");

const CsvController = require("./controllers/CsvController");
const PdfController = require("./controllers/PdfController");
const DateController = require("./controllers/DateController");
const WifiController = require("./controllers/WifiController");
const EditController = require("./controllers/EditController");
const BackupController = require("./controllers/BackupController");
const UpdateController = require("./controllers/UpdateController");
const TemperatureController = require("./controllers/TemperatureController");

class WLDAPI {

    constructor() {

        this.controllers = {
            'test': (args) => WLDWindow.test(args),
            'renderer': (args) => WLDWindow.renderer(args),

            'print':                     (args) => WLDMain.addPrint(args),

            'ALERT/Close':               (args) => WLDAlert.close(args),

            'ALERT/Confirm':             (args) => WLDMain.alertConfirmPressed(args),
            'ESSAY/begin':               (args) => WLDMain.startTeste(args),
            'clickOnSlot':               (args) => WLDMain.clickOnSlot(args),
            'readImage':                 (args) => WLD.readImageOnDisk(args),
            'DATE/change':               (args) => WLDMain.datedate(args),
            'SOUND/change':              (args) => WLDMain.changeSound(args),
            'QRCODE/change':             (args) => WLDMain.printQRcode(args),
            'CALIBRATE/on':              (args) => WLDMain.getKey(args),
            'LANGUAGE/render':           (args) => WLDMain.verifyLanguage(args),
            'LANGUAGE/confirm':          (args) => WLDMain.setLanguage(args),
            'KEY/Confirm':               (args) => WLDMain.verifyKey(args),
            'PRINTER/Test':              (args) => WLDMain.printerTesting(args),

            'closeNotification':         (args) => WLDNotifications.remove(),
            
            'WIFI/Connect':              (args) => WifiController.connect(args),
            'WIFI/Clear':                (args) => WifiController.delete(args),
            
            'RENDER/DATE':               (args) => DateController.Update(args),

            'RENDER/EDIT':               (args) => EditController.open(args),
            'RENDER/WIFI':               (args) => WifiController.open(args),
            'enable/dissable/wifi':      (args) => WifiController.enable(args),

            // 'RENDER/REPORT': (args) => WLDMain.open(args),
            'openReport':                (args) => WLDMain.openReport(args),
            
            'RENDER/ESSAYS':             (args) => WLDMain.openEssays(args),
            // 'RENDER/ESSAYS': (args) => WLDMain.open(args),
            
            'RENDER/config':             (args) => WLDMain.openConfig(args),
            // 'RENDER/CONFIG': (args) => WLDMain.open(args),
            
            'RENDER/RESULT':             (args) => WLDMain.closeResult(args),


            'UPDATE':                    (args) => UpdateController.checkUpdate(),

            'TEMPERATURE/select':        (args) => TemperatureController.Select(args),
            'TEMPERATURE/change':        (args) => TemperatureController.Change(args),

            'EDIT/Input/alert':          (args) => EditController.erroEdit(args),
            'EDIT/add':                  (args) => EditController.editAdd(args),
            'EDIT/list':                 (args) => EditController.editList(args),
            'EDIT/verify':               (args) => EditController.editVerify(args),
            'EDIT/edit':                 (args) => EditController.editChange(args),
            'EDIT/del':                  (args) => EditController.editDelete(args),

            'pdf':                       (args) => PdfController.makePDF(args),
            'DATABASE/Export/PDF':       (args) => PdfController.exportPDFs(args),

            'openWindowBackup':          (args) => BackupController.automaticBackup(args),

            'BACKUPAUTOMATIC':           (args) => BackupController.setBackup(args),
            'DATABASE/Export':           (args) => BackupController.exportDatabase(args),
            'DATABASE/Import/Selected':  (args) => BackupController.importConfirmDatabase(args),

            'DATABASE/Export/CSV':       (args) => CsvController.geraCSV(args),

            'DATABASE/Clear':            (args) => WLDMain.getKey(args),
            'DATABASE/Import':           (args) => WLDMain.getKey(args),
        }
    }

}

module.exports = new WLDAPI();