const WLDIO = require("../WLD/WLDIO");
const WLDBD = require('../WLD/WLDDatabase');
const WLDConfig = require('../WLD/WLDConfig');

const { WLD } = require("../WLD/WLDUtils");
const WLDSerial = require('../WLD/WLDSerial');
const WLDNotifications = require('../WLD/WLDNotifications');

const { Alerta } 	= require("../WLD/WLDAlert");
const { renderer }  = require("../WLD/WLDWindow");
const { translate } = require("../WLD/WLDLanguage");

const DatabaseController = require('./DatabaseController');

class BackupController {

	constructor() {
		this.today = 0;
		this.flagCanBackup = true;
	}

    importConfirmDatabase = async (args) => {

		renderer().close('settings', 'selection');

		const folder = await WLDIO.verifyPendrive();

		if(!folder) {
			Alerta('pdf', translate(97), translate(26), null, translate(11));
			return WLD.log('IMPORT', 'Nenhum pendrive encontrado');
		}

		const folderPath = `/media/pi/${folder}/lummina pro 8 Backup's/`;
		const folderName = args['folder'];

		const pathDatabase = WLDBD.database_location;
		const newLocation  = folderPath + folderName;
		const photosPath   = '/home/pi/wld/pics';

		Alerta('pdf', translate(159), translate(6));
		WLD.log('IMPORT', 'importando backup.');

		const folder_exists = await WLDIO.verifyFolder(folderPath);

		if(!folder_exists) {
			Alerta('pdf', translate(160), translate(26), null, translate(11));
			return WLD.log('IMPORT', 'pasta de backups não encontrada.');
		}

		const resultFile = await WLDIO.copyFile(newLocation + '/database.db', pathDatabase);
				
		if (!resultFile) {
			Alerta('pdf', translate(161), translate(26), null, translate(11));
			return WLD.log('IMPORT', 'erro ao importar arquivo.');
		}

		const resultPhotosFolder = await WLDIO.copyFolder(newLocation + '/pics', photosPath);
				
		if (!resultPhotosFolder) {
			Alerta('pdf', translate(162), translate(26), null, translate(11));
			return WLD.log('IMPORT', 'erro ao importar pasta.');
		}

		WLDIO.RemoveDotFiles();

		Alerta('pdf', translate(163), translate(103), null, translate(11));
		WLD.log('IMPORT', 'importação do backup realizada.');
	}

	importDatabase = async () => {

		const folder = await WLDIO.verifyPendrive();

		if(!folder) {
			Alerta('pdf', translate(97), translate(26), null, translate(11));
			return WLD.log('IMPORT', 'Nenhum pendrive encontrado');
		}

		const folderPath = `/media/pi/${folder}/lummina pro 8 Backup's/`;

		const folder_exists = await WLDIO.verifyFolder(folderPath);

		if(!folder_exists) {
			Alerta('pdf', 'Pasta de backups não encontrada', translate(26), null, translate(11));
			return WLD.log('IMPORT', 'pasta de backups não encontrada.');
		}

		const folders = await WLDIO.listDir(folderPath);

		let folderFilter = [];

		folders.map((item) => (item.includes('BK_') && !item.includes('.')) && folderFilter.push(item));

		folderFilter.sort((a, b) => parseInt(b.substring(3)) - parseInt(a.substring(3)));

		renderer().open('settings', 'selection', folderFilter);
	}

	exportDatabase = async (notify) => {
		const state = await WLDSerial.checkState();
		const folder = await WLDIO.verifyPendrive();

		if (!state) {
			Alerta('temperature', translate(152), translate(26), null, translate(11), notify);
			return WLD.log('EXPORT', 'não é possível exportar com testes em andamento.');
		}

		if(!folder) {
			if (!notify) WLDNotifications.add({ id: 97, date: `- ${WLD.getTime('hhhmmm')}`, icon: ''  });
			Alerta('pdf', translate(97), translate(26), null, translate(11), notify);
			return WLD.log('EXPORT', 'Nenhum pendrive encontrado');
		}

		const folderPath = `/media/pi/${folder}/lummina pro 8 Backup's/`;
		const fileName = `BK_${WLD.getDate('yyyymmdd')}${WLD.getTime('hhmmss')}/`;

		const pathDatabase = WLDBD.database_location;
		const newLocation = folderPath + fileName;
		const photosPath = '/home/pi/wld/pics';

		Alerta('pdf', translate(104), translate(6), null, null, notify);
		WLD.log('EXPORT', 'realizando backup.');

		const args = { newLocation, pathDatabase, photosPath, notify }

		const folder_exists = await WLDIO.verifyFolder(folderPath);

		if(!folder_exists) {
			const folder_created = await WLDIO.createFolder(folderPath);
			
			if(!folder_created) {
				Alerta('pdf', 'Problema com a pasta', translate(26), null, translate(11), notify);
				return WLD.log('EXPORT', 'problema com a pasta.');
			}
			
			exportDB(args);
		}

		exportDB(args);

		return true;
	}
	
	clearDatabase = () => Alerta('clearDatabase', translate(101), translate(8), translate(32), translate(31));

	clearDatabaseConfirm = async () => {
		DatabaseController.Clear();

		Alerta('clearDatabase', translate(136), translate(103), null, translate(11));
		WLD.log('BANCO DE DADOS', 'banco de dados limpo com sucesso.');
	}

	automaticBackup() {
		const isON = WLDConfig.read('backup', 'isON', true);
		const time = WLDConfig.read('backup', 'time', '00:00');
		const last_time = WLDConfig.read('backup', 'last_time', '');

		const hour = time.substring(0,2);
		const minute = time.substring(3,5);

		renderer().open('settings', 'backup', { isON, last_time, time: { hour, minute }});
	}

	setBackup({ isON, time: { hour, minute } }) {
		WLDConfig.write('backup', 'isON', isON);
		WLDConfig.write('backup', 'time', `${hour}:${minute}`);
		WLDConfig.write('backup', 'canBackup', true);
	}

	todayHasChanged() {
		const date = new Date();
		let day = date.getDate().toString().padStart(2, 0);
		
		if(this.today !== day) {
			this.today = day;
			WLDConfig.write('backup', 'canBackup', true);
			return true;
		}

		return false;
	}

	async loop() {
		const isON = WLDConfig.read('backup', 'isON', true);

		this.flagCanBackup = WLDConfig.read('backup', 'canBackup', true);;

		if(this.todayHasChanged()) this.flagCanBackup = true;

		if(isON && this.flagCanBackup) {
			const currentTime = WLD.getTime('hh:mm');
			const timeForBackup = WLDConfig.read('backup', 'time', '00:00');

			if(currentTime == timeForBackup) {
				this.flagCanBackup = false;
				const doneBackup = await this.exportDatabase(false);
				WLDConfig.write('backup', 'canBackup', false);

				if(doneBackup) WLDConfig.write('backup', 'last_time', `${WLD.getTime('hh : mm')} - ${WLD.getDate('dd/mm/yyyy')}`);
			}
		}
	}

}

async function exportDB(args) {

	const { newLocation, pathDatabase, photosPath, notify } = args;

	const folder_file_created = await WLDIO.createFolder(newLocation);
			
	if(!folder_file_created) {
		Alerta('pdf', 'Problema com a pasta', translate(26), null, translate(11), notify);
		return WLD.log('EXPORT', 'problema com a pasta.');
	}

	const resultFile = await WLDIO.copyFile(pathDatabase, newLocation + 'database.db');
		
	if (!resultFile) {
		Alerta('pdf', 'Erro ao gravar arquivo', translate(26), null, translate(11), notify);
		return WLD.log('EXPORT', 'erro ao gravar arquivo.');
	}

	const resultEnsaiosFolder = await WLDIO.copyFolder(photosPath + '/ensaios', newLocation + '/pics/ensaios');
			
	if (!resultEnsaiosFolder) {
		Alerta('pdf', 'Erro ao copiar pasta', translate(26), null, translate(11), notify);
		return WLD.log('EXPORT', 'erro ao copiar pasta.');
	}

	const resultPrintsFolder = await WLDIO.copyFolder(photosPath + '/print', newLocation + '/pics/print');
			
	if (!resultPrintsFolder) {
		Alerta('pdf', 'Erro ao copiar pasta', translate(26), null, translate(11), notify);
		return WLD.log('EXPORT', 'erro ao copiar pasta.');
	}

	if (!notify) WLDNotifications.add({ id: 102, date: `- ${WLD.getTime('hhhmmm')}`, icon: ''  });
	Alerta('pdf', translate(102), translate(103), null, translate(11), notify);
	return WLD.log('EXPORT', 'backup realizado.');
}

module.exports = new BackupController();