const { WLD } = require("../WLD/WLDUtils");
const WLDIO = require("../WLD/WLDIO");
const WLDSerial = require('../WLD/WLDSerial');
const WLDDatabase = require('../WLD/WLDDatabase');

const { Alerta } 	= require("../WLD/WLDAlert");
const { translate } = require("../WLD/WLDLanguage");

const glBD = WLDDatabase;

class CsvController {

	geraCSV = async () => {

        const state = WLDSerial.checkState();
    
        if (!state) {
            Alerta('temperature', translate(152), translate(26), null, translate(11));
            return WLD.log('CSVs', 'não é possível exportar com testes em andamento.');
        }
    
        const folder = await WLDIO.verifyPendrive();
    
        if (!folder) {
            Alerta('pdf', translate(97), translate(26), null, translate(11));
            return WLD.log('CSVs', translate(97));
        }
    
        const data = await glBD.getDatas('csv_essays');
    
        if(data.length == 0) {
            Alerta('pdf', translate(150), translate(26), null, translate(11));
            return WLD.log('CSVs', 'sem registros para exportar.');
        }

        const columnOrder = ["44", "45", "126", "48", "56", "108", "47", "125","60", "46", "49", "110"];

        const header = columnOrder.map(col => translate(col * 1)).join(';') + '\n';

        const outData = data.reduce((acc, row) => {
            const orderedValues = columnOrder.map(col => {
                if (["48", "56", "49", "60"].includes(col)) {
                    return JSON.stringify(translate(row[col]));
                }

                return JSON.stringify(row[col]);
            });

            return acc + orderedValues.join(';') + '\n';

        }, header);
    
        const _data = {
            outData,
            fileType: 'csv',
            createFile: 'Gerando CSV',
            fileFolder: "lummina pro 8 CSV's",
            successMsg: translate(146),
        };
    
        this.gravaArquivo(_data);
        
    }
    
    gravaArquivo = async (args) => {
    
        const folder = await WLDIO.verifyPendrive();
    
        if (!folder) {
            Alerta('pdf', translate(97), translate(26), null, translate(11));
            return WLD.log('CSVs', translate(97));
        }
    
        const { successMsg, outData, createFile, fileType, fileFolder } = args;
    
        const folderPath = `/media/pi/${folder}/${fileFolder}/`;
        const filePath = `${folderPath}${WLD.getDate('yyyymmdd')}${WLD.getTime('hhmmss')}.${fileType}`;
    
        const _data = { outData, filePath };
    
        Alerta('pdf', createFile, translate(6));
        WLD.log('CSVs', 'gerando csv.');
    
        const folder_exists = await WLDIO.verifyFolder(folderPath);
    
        if(!folder_exists) {
            const folder_created = await WLDIO.createFolder(folderPath);
    
            if(!folder_created) {
                Alerta('pdf', 'Problema com a pasta', translate(26), null, translate(11));
                return WLD.log('CSVs', 'problema com a pasta.');
            }
                
            const result = await WLDIO.writeFile(_data);
                
            if (!result) {
                Alerta('pdf', 'Erro ao gravar arquivo', translate(26), null, translate(11));
                return WLD.log('CSVs', 'erro ao gravar arquivo.');
            }
                
            Alerta('pdf', successMsg, translate(103), null, translate(11));
            return WLD.log('CSVs', 'csv gerado com sucesso.');
        }
        
        const result = await WLDIO.writeFile(_data);
    
        if (!result) {
            Alerta('pdf', 'Erro ao gravar arquivo', translate(26), null, translate(11));
            return WLD.log('CSVs', 'erro ao gravar arquivo.');
        }
                
        Alerta('pdf', successMsg, translate(103), null, translate(11));
        WLD.log('CSVs', 'csv gerado com sucesso.');
    }
}

module.exports = new CsvController();