const WLDIO 	  = require("../WLD/WLDIO");
const WLDPdf 	  = require("../WLD/WLDPdf");
const WLDCamera   = require('../WLD/WLDCamera');
const WLDDatabase = require('../WLD/WLDDatabase');

const { WLD } = require("../WLD/WLDUtils");
const WLDSerial = require("../WLD/WLDSerial");

const { Alerta }    = require("../WLD/WLDAlert");
const { translate } = require("../WLD/WLDLanguage");

const glPdf = new WLDPdf();
const glCam = new WLDCamera();
const glBD = WLDDatabase;

class PdfController {

	constructor() { this.isExporting = false; }

	makePDF = async (id, isAll = false, current, max, isLast) => {
		const folder = await WLDIO.verifyPendrive();


		if(!folder) {
			Alerta('pdf', translate(97), translate(26), null, translate(11));
			return WLD.log('PDF', translate(97));
		}

		const folderPath = `/media/pi/${folder}/lummina pro 8 PDF's/`;

		if (isAll) {
			Alerta('pdf', `${translate(144)} ${current} ${translate(145)} ${max}`, translate(6));
			WLD.log('PDF', `gerando pdf ${current} de ${max}.`);
		} else {
			Alerta('pdf', translate(144), translate(6));
			WLD.log('PDF', 'gerando pdf.');
		}
		
		const folder_exists = await WLDIO.verifyFolder(folderPath);

		if(!folder_exists) {
			const folder_created = await WLDIO.createFolder(folderPath);
			
			if(!folder_created) {
				if(!isAll) Alerta('pdf', 'Problema com a pasta', translate(26), null, translate(11));
				return WLD.log('PDF', 'problema com a pasta.');
			}
			
			const pdf_create = await glPdf.makePdf(id, folder);
			
			if(!pdf_create) {
				if(!isAll) Alerta('pdf', 'Erro ao gerar arquivo', translate(26), null, translate(11));
				return WLD.log('PDF', 'erro ao gerar arquivo.');
			}

			if(isLast) Alerta('pdf', `${max} ${translate(142)}`, translate(103), null, translate(11));
			if(!isAll) Alerta('pdf', translate(143), translate(103), null, translate(11));
			return WLD.log('PDF', 'pdf gerado com sucesso.');
		}

		const pdf_create = await glPdf.makePdf(id, folder);
			
		if(!pdf_create) {
			if(!isAll) Alerta('pdf', 'Erro ao gerar arquivo', translate(26), null, translate(11));
			return WLD.log('PDF', 'erro ao gerar arquivo.');
		}

		if(isLast) Alerta('pdf', `${max} ${translate(142)}`, translate(103), null, translate(11));
		if(!isAll) Alerta('pdf', translate(143), translate(103), null, translate(11));
		return WLD.log('PDF', 'pdf gerado com sucesso.');
	}
	
	exportPDFs = async () => {
		if(!this.isExporting) {
			this.isExporting = true;

			const state = await WLDSerial.checkState();

			if (!state) {
				this.isExporting = false;
				Alerta('temperature', translate(152), translate(26), null, translate(11));
				return WLD.log('PDFs', 'não é possível exportar com testes em andamento.');
			}

			const folder = await WLDIO.verifyPendrive();

			if (!folder) {
				this.isExporting = false;
				Alerta('pdf', translate(97), translate(26), null, translate(11));
				return WLD.log('PDFs', translate(97));
			}

			const ensaios = await glBD.getDatas('all_pdfs');

			if(ensaios.length == 0) {
				this.isExporting = false;
				Alerta('pdf', translate(150), translate(26), null, translate(11));
				return WLD.log('PDFs', 'sem registros para exportar.');
			}

			for (let index = 0; index < ensaios.length; index++) {
				glCam.convertPhoto(ensaios[index]['N INCUB']);
				await WLD.delay(1500);
				this.makePDF(ensaios[index]['N INCUB'], true, index + 1, ensaios.length, (index == ensaios.length - 1) && true);
			}

			this.isExporting = false;
		}
	}
}

module.exports  = new PdfController();