export default function Settings(props) {

    const { 
        _key, 
        _render, 
        _edit, 
        _input, 
        _menu, 
        _wifi, 
        _config,
        _date,
        _calibrate, 
        _backup_selection, 
        _backup_automatic,
        _guide
    } = props;

    const translate = window.language.translate

    const _settings = {
        date: ({ visible, data: { current = {}} = {}}) => { 
            if(!visible) return _render.enableDate(visible);

            _date.setCurrent(current);
            _render.enableDate(visible);
            _render.enableMenu(!visible);
        },
        wifi: ({ visible, data: { list = [] } = {}}) => { 

            if(!visible) return _render.enableWifi(visible);

            _wifi.setNetworks(list);
            _wifi.setWifiPassword('');
            _wifi.setWifiSelected(list[0]);

            _render.enableWifi(visible);
            _render.enableMenu(!visible);
        },
        language: ({ visible }) => {
            _render.enableLanguage(visible);
            _render.enableConfig(false);
        },
        input: ({ visible, data: { type = '', mode = '', info = '' } = {}}) => { 

            if(!visible) return _render.enableInput(visible);

            _render.enableInput(visible);

            _input.setModeInput(mode);
            _input.setCurrentId(info['id']);

            if (mode == 'editing') {
                if (type === 'users') {
                    _input.setInputUserTextOne(info['name']);
                    _input.setInputUserSelected(info['role']);
                } else {
                    _input.setInputEquipTextOne(info['name']);
                    _input.setInputEquipTextTwo(info['role']);
                }
            } else {
                _input.setInputUserTextOne('');
                _input.setInputUserTextTwo('');
                _input.setInputEquipTextOne('');
                _input.setInputEquipTextTwo('');
            }

            if (type === 'users') {
                _input.setInputEditing('users');
            } else {
                _input.setInputEditing('equips');
            }

            if (mode === 'adding') {
                _input.setInputHeader(translate(19));
            } else {
                _input.setInputHeader(translate(141));
            }

            // _edit.setEditArray(prev => prev);
        },
        edit: ({ visible, data, data: { list = [], header = '' } = {}}) => {
            if(!visible) return _render.enableEdit(visible);

            console.log('entrou no edit' + JSON.stringify(data));

            var newArray = []

            list.map(item => {
                if (header === 'users') {
                    newArray.push({ id: item['id_user'], name: item['name'], role: item['type'] });
                } else {
                    if (item['model'] === 'CONTROLE') return;
                    newArray.push({ id: (item['id_equip'] - 1), name: item['model'], role: item['brand'] });
                }
            });

            (header === 'users') 
                ? _edit.setEditHeader({ id: translate(13), name: translate(14), role: 'ACESSO' })
                : _edit.setEditHeader({ id: translate(13), name: translate(25), role: 'MARCA' });

            _edit.setEditType(header);
            _edit.setEditArray(newArray);
            
            _render.enableMenu(!visible);
            _render.enableEdit(visible);
        },
        config: (data) => {
            const { visible, data: { info = {}, print_qr = false} = {}} = data;

            if (!visible) return _render.enableConfig(visible);

            _config.setConfigModel(info['model']);
            _config.setConfigSerialNumber(info['serial_number']);
            _config.setConfigVersionSoftware(info['version_software']);
            _config.setConfigVersionFirmware(info['version_firmware']);
            _config.setConfigFabricationDate(info['fabrication_date']);
            _config.setSwitchValue(print_qr);
            _render.enableMenu(!visible);
            _render.enableConfig(visible);
        },
        selection: (data) => {
            const { visible, data: folder = [] } = data;

            if(!visible) return _render.enableBackupSelection(visible);

            _backup_selection.setFolders(folder);
            _backup_selection.setFolderSelected(folder[0]);
            _render.enableBackupSelection(visible);
        },
        key: (data) => {
            const { visible, data: { key = '', type = '' } = {}} = data;

            if(!visible) return _render.enableKey(visible);

            _key.setKey(key);
            _key.setType(type);
            _render.enableKey(visible);
            _render.enableMenu(!visible);
        },
        calibrate: ({ visible, data: { offset = 0 } = {}}) => {
            _calibrate.setOffset(parseInt(offset));
            _render.enableCalibrate(visible);
        },
        backup: ({ visible, data: { isON = true, last_time = '', time = {}} = {}}) => {

            _backup_automatic.setIsON(isON);
            _backup_automatic.setTime(time);
            _backup_automatic.setLastTime(last_time);

            _render.enableBackupAutomatic(visible);
        },
        guide: ({ visible }) => {
            _guide.setIsFirstSlide(true);
            _guide.setIsLastSlide(false);
            _guide.setCurrentSlide(0);
            _render.enableGuide(visible);
        }
    }

    return _settings;
}