import Render from './exports/render';

import Popup from './exports/popup';
import Screen from './exports/screen';
import Settings from './exports/settings';

import KeyContainer from './containers/KeyContainer';
import ZoomContainer from './containers/ZoomContainer';
import WifiContainer from './containers/WifiContainer';
import EditContainer from './containers/EditContainer';
import ShowContainer from './containers/ShowContainer';
import MenuContainer from './containers/MenuContainer';
import DateContainer from './containers/DateContainer';
import InputContainer from './containers/InputContainer';
import StartContainer from './containers/StartContainer';
import AlertContainer from './containers/AlertContainer';
import ConfigContainer from './containers/ConfigContainer';
import BackupContainer from './containers/BackupContainer';
import EssaysContainer from './containers/EssaysContainer';
import ReportContainer from './containers/ReportContainer';
import ResultContainer from './containers/ResultContainer';
import CapsuleContainer from './containers/CapsuleContainer';
import KeyboardContainer from './containers/KeyboardContainer';
import CalibrateContainer from './containers/CalibrateContainer';
import SelectionContainer from './containers/SelectionContainer';
import GuideContainer from './containers/GuideContainer';

export default function Manager() {
    
    const _render = Render();

    const _guide = GuideContainer();
    const _key = KeyContainer();
    const _date = DateContainer();
    const _zoom = ZoomContainer();
    const _wifi = WifiContainer();
    const _edit = EditContainer();
    const _show = ShowContainer();
    const _menu = MenuContainer();
    const _cap = CapsuleContainer();
    const _input = InputContainer();
    const _start = StartContainer();
    const _alerta = AlertContainer();
    const _backup = BackupContainer();
    const _essays = EssaysContainer();
    const _report = ReportContainer();
    const _result = ResultContainer();
    const _config = ConfigContainer();
    const _keyboard = KeyboardContainer();
    const _calibrate = CalibrateContainer();
    const _selection = SelectionContainer();
    
    const _backup_selection = _backup['_selection'];
    const _backup_automatic = _backup['_automatic'];

    const __popup = { _render, _show, _essays, _zoom, _result, _alerta, _start, _selection, _menu };
    const __screen = { _render, _report, _keyboard, _input, _wifi, _start };
    const __settings = { _render, _edit, _input, _menu, _wifi, _config, _backup_selection, _key, _calibrate, _backup_automatic, _date, _guide };
    
    const _popup = Popup(__popup);
    const _screen = Screen(__screen);
    const _settings = Settings(__settings);

    const _application = {
        _key: _key['_data'],
        _caps: _cap['_data'],
        _zoom: _zoom['_data'],
        _wifi: _wifi['_data'],
        _edit: _edit['_data'],
        _show: _show['_data'],
        _menu: _menu['_data'],
        _slots: _cap['_slots'],
        _start: _start['_data'],
        _input: _input['_data'],
        _config: _config['_data'],
        _render: _render['_data'],
        _report: _report['_data'],
        _result: _result['_data'],
        _essays: _essays['_data'],
        _keyboard: _keyboard['_data'],
        _selection: _selection['_data'],
        _calibrate: _calibrate['_data'],
        _guide: _guide['_data'],
        _date: _date['_data'],
        
        _backup_automatic: _backup_automatic['_data'],
        _backup_selection: _backup_selection['_data'],

        _alert: {
            _pdf: _alerta['_pdf'],
            _wifi: _alerta['_wifi'],
            _edit: _alerta['_edit'],
            _camera: _alerta['_camera'],
            _printer: _alerta['_printer'],
            _temperature: _alerta['_temperature'],
            _database: { _clear: _alerta['_databaseClear'] },
        },

        _renderScreen: {
            popup: _popup,
            screen: _screen,
            settings: _settings,
        },
    }

    return _application;
}