const { contextBridge, ipcRenderer } = require("electron");

const WLDLanguage = require('./WLD/WLDLanguage');
const WLDConfig = require('./WLD/WLDConfig');

WLDLanguage.init();

let hold_zoom;
let hold_result;
let hold_processo;

const _results = {
	0: 51, // Em Andamento,
	1: 54, // Positivo,
	2: 55, // Negativo,
	3: 52, // Cancelado, 
	99: ''
}

function calculate({ remaining, total }) {
    const difference = total - remaining;

    const percentage = Math.trunc((difference * 100) / total);
    const minutes    = Math.floor((difference / (1000 * 60)) % 60).toString().padStart(2, 0);
    const hours      = Math.floor((difference / (1000 * 60   * 60)) % 24).toString().padStart(2, 0);
    
    return { percentage, minutes, hours };
}

function renderer(args) {
    const close  = (mode, type)            => ipcRenderer.send('api', { controller: 'renderer', args: { mode, type, visible: false } });
    const open   = (mode, type, data = {}) => ipcRenderer.send('api', { controller: 'renderer', args: { mode, type, visible: true, data } });

    if(args) return ipcRenderer.send('api', { controller: 'renderer', args });

    return { open, close }
}


const API = {
    renderer,
    test: (args) => ipcRenderer.send('api', { controller: 'test', args }),
    wait: async (channel, args)     => ipcRenderer.invoke(channel, args),
    close:      (channel)           => ipcRenderer.removeListener(channel, () => {}),
    update:     (channel, callback) => ipcRenderer.on(channel, (_, args) => callback(args)),
    send:       (channel, args)     => ipcRenderer.send('api', { controller: channel, args }),
}

const LANGUAGE = {
    translate: (id) => WLDLanguage.translate(id),
    change: (language) => WLDLanguage.setCurrent(language),
}

const IO = {
    read: (type, name, value) => WLDConfig.read(type, name, value),
    write: (type, name, value) => WLDConfig.write(type, name, value), 
    calibrate: (value) => WLDConfig.write('calibrate', 'temperature_offset', value), 
}

const MATH = {
    calculate: (value) => calculate(value)
}

const VARIABLES = {
    set_hold_zoom:     (value) => hold_zoom = value,
    set_hold_result:   (value) => hold_result = value,
    set_hold_processo: (value) => hold_processo = value,
    get_hold_zoom:     () => hold_zoom,
    get_hold_result:   () => hold_result,
    get_hold_processo: () => hold_processo,
    results: _results,
}

contextBridge.exposeInMainWorld('io', IO);
contextBridge.exposeInMainWorld('api', API);
contextBridge.exposeInMainWorld('math', MATH);
contextBridge.exposeInMainWorld('language', LANGUAGE);
contextBridge.exposeInMainWorld('variables', VARIABLES);